#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file EditSlots.py
# user pool commands of GUI application

""" Edit slots """

from PyQt5.QtGui import QKeySequence
from PyQt5.QtCore import Qt


from .EditCommands import (
    ComponentEdit,
    DataSourceCopy,
    DataSourceCut,
    DataSourcePaste,
    DataSourceApply,
    DataSourceEdit,
    ComponentTakeDataSources,
    ComponentTakeDataSource
)


# stack with the application commands
class EditSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # action data
        self.actions = {
            "actionEditComponent": [
                "&Edit Component", "componentEdit",
                "Ctrl+E", "componentedit", "Edit the component"],
            "actionTakeDataSourceItem": [
                "Take DataSource Item ", "componentTakeDataSource",
                "Ctrl+G",
                "componenttakedatasource",
                "Take the currnet data sources from the component"],
            "actionTakeDataSources": [
                "Take DataSources ", "componentTakeDataSources",
                "",
                "componenttakedatasource",
                "Take data sources from the component"],
            "actionEditDataSource": [
                "&Edit DataSource", "dsourceEdit",
                "Ctrl+Shift+E",
                "dsourceedit", "Edit the data source"],
            "actionApplyDataSource": [
                "Apply DataSource", "dsourceApply",
                "Ctrl+Shift+R", "dsourceapply", "Apply the data source"],
            "actionCopyDataSource": [
                "Copy DataSource", "dsourceCopy",
                "", "copy", "Copy the data source"],
            "actionCutDataSource": [
                "Cut DataSource", "dsourceCut",
                QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_Delete),
                "cut", "Cut the data source"],
            "actionPasteDataSource": [
                "Paste DataSource", "dsourcePaste",
                QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_Insert),
                "paste", "Paste the data source"]
        }

    # take datasources
    # \brief It takes datasources from the current component
    def componentTakeDataSources(self):
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentTakeDataSources(self.main)
        cmd.redo()
        self.undoStack.clear()

    # take datasources
    # \brief It takes datasources from the current component
    def componentTakeDataSource(self):
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentTakeDataSource(self.main)
        self.undoStack.push(cmd)

    # edit component action
    # \brief It opens a dialog with the current component
    def componentEdit(self, _=None):
        cmd = ComponentEdit(self.main)
        cmd.redo()

    # edit datasource action
    # \brief It opens a dialog with the current datasource
    def dsourceEdit(self, _=None):
        cmd = DataSourceEdit(self.main)
        cmd.redo()

    # apply datasource item action executed by button
    # \brief It applies the changes in the current datasource item
    #        executed by button
    def dsourceApplyButton(self):
        if self.main.updateDataSourceListItem():
            self.dsourceApply()

    # apply datasource item action
    # \brief It applies the changes in the current datasource item
    def dsourceApply(self):
        cmd = DataSourceApply(self.main)
        self.undoStack.push(cmd)

    # copy datasource item action
    # \brief It copies the  current datasource item into the clipboard
    def dsourceCopy(self):
        cmd = DataSourceCopy(self.main)
        cmd.redo()

    # cuts datasource item action
    # \brief It removes the current datasources item and copies it
    #        into the clipboard
    def dsourceCut(self):
        cmd = DataSourceCut(self.main)
        self.undoStack.push(cmd)

    # paste datasource item action
    # \brief It pastes the datasource item from the clipboard
    def dsourcePaste(self):
        cmd = DataSourcePaste(self.main)
        self.undoStack.push(cmd)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ServerSlots.py
# user pool commands of GUI application

""" Server slots """


from .ServerCommands import (
    ServerConnect,
    ServerCPCreate,
    ServerSCPCreate,
    ServerDSCreate,
    ServerFetchComponents,
    ServerStoreComponent,
    ServerStoreAllComponents,
    ServerDeleteComponent,
    ServerSetMandatoryComponent,
    ServerGetMandatoryComponents,
    ServerUnsetMandatoryComponent,
    ServerFetchDataSources,
    ServerStoreDataSource,
    ServerStoreAllDataSources,
    ServerDeleteDataSource,
    ServerClose)

from .EditCommands import (
    ComponentEdit,
    DataSourceApply,
    DataSourceEdit
)

from .ItemCommands import (
    ComponentApplyItem,
    ComponentMerge,
)


# stack with the application commands
class ServerSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # action data
        self.actions = {
            "actionConnectServer": [
                "&Connect ...", "serverConnect",
                "Ctrl+T", "serverconnect",
                "Connect to the configuration server"],
            "actionFetchComponentsServer": [
                "&Fetch Components", "serverFetchComponents",
                "Ctrl+F", "serverfetchdatasources",
                "Fetch datasources from the configuration server"],

            "actionStoreComponentServer": [
                "&Store Component", "serverStoreComponent",
                "Ctrl+B", "serverstorecomponent",
                "Store component in the configuration server"],

            "actionStoreAllComponentsServer": [
                "&Store All Components", "serverStoreAllComponents",
                "", "serverstoreallcomponents",
                "Store all components in the configuration server"],

            "actionDeleteComponentServer": [
                "&Delete Component", "serverDeleteComponent",
                "Ctrl+H", "serverdeletecomponent",
                "Delete component from the configuration server"],

            "actionFetchDataSourcesServer": [
                "&Fetch DataSources", "serverFetchDataSources",
                "Ctrl+Shift+F", "serverfetchdatasources",
                "Fetch datasources from the configuration server"],

            "actionStoreDataSourceServer": [
                "&Store Datasource", "serverStoreDataSource",
                "Ctrl+Shift+B", "serverstoredatasource",
                "Store datasource in the configuration server"],

            "actionStoreAllDataSourcesServer": [
                "&Store All Datasources", "serverStoreAllDataSources",
                "", "serverstorealldatasources",
                "Store all datasources in the configuration server"],

            "actionDeleteDataSourceServer": [
                "&Delete Datasource", "serverDeleteDataSource",
                "Ctrl+Shift+H", "serverdeletedatasource",
                "Delete datasource from the configuration server"],

            "actionSetComponentMandatoryServer": [
                "Set Component Mandatory", "serverSetMandatoryComponent",
                "", "serversetmandatory",
                "Set the component as mandatory  on the configuration server"],

            "actionGetMandatoryComponentsServer": [
                "Get Mandatory Components", "serverGetMandatoryComponents",
                "", "servergetmandatory",
                "Get mandatory components  from the configuration server"],

            "actionUnsetComponentMandatoryServer": [
                "Unset Component Mandatory", "serverUnsetMandatoryComponent",
                "", "serverunsetmandatory",
                "Unset the component as mandatory on"
                " the configuration server"],
            "actionCreateStdComponentServer": [
                "&Create Standard Component ...", "serverSCPCreate",
                "", "serverscpcreate",
                "Create Component defined in online.xml file"],
            "actionCreateComponentServer": [
                "&Create Online Component ...", "serverCPCreate",
                "", "servercpcreate",
                "Create Component defined in online.xml file"],
            "actionCreateDataSourcesServer": [
                "&Create Online Component ...", "serverDSCreate",
                "", "serverdscreate",
                "Create all known DataSources defined in online.xml file"],
            "actionCloseServer": [
                "C&lose", "serverClose",
                "Ctrl+L", "serverclose",
                "Close connection to the configuration server"]
        }

    # connect server action
    # \brief It connects to configuration server
    def serverConnect(self):
        cmd = ServerConnect(self.main)
        self.undoStack.push(cmd)

    # create component action
    # \brief It creates components and datasources from online.xml
    def serverCPCreate(self):
        cmd = ServerCPCreate(self.main)
        cmd.redo()
        self.undoStack.clear()

    # create standard component action
    # \brief It creates stamdard components and datasources from online.xml
    def serverSCPCreate(self):
        cmd = ServerSCPCreate(self.main)
        cmd.redo()
        self.undoStack.clear()

    # create datasources action
    # \brief It creates all known datasources from online.xml
    def serverDSCreate(self):
        cmd = ServerDSCreate(self.main)
        cmd.redo()
        self.undoStack.clear()

    # fetch server components action
    # \brief It fetches components from the configuration server
    def serverFetchComponents(self):
        cmd = ServerFetchComponents(self.main)
        cmd.redo()
        self.undoStack.clear()

    # store server component action executed by button
    # \brief It stores the current component
    #        in the configuration server executed by button
    def serverStoreComponentButton(self):
        if self.main.updateComponentListItem():
            cmd = ComponentApplyItem(self.main)
            self.undoStack.push(cmd)
            self.serverStoreComponent(False)

    # store server component action
    # \brief It stores the current component in the configuration server
    def serverStoreComponent(self, focus=True):
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentMerge(self.main)
        self.undoStack.push(cmd)
        cmd = ServerStoreComponent(self.main)
        cmd.redo()
        if focus:
            self.main.componentList.setItemFocus()

    # store server all components action
    # \brief It stores all components in the configuration server
    def serverStoreAllComponents(self):
        cmd = ComponentApplyItem(self.main)
        cmd = ServerStoreAllComponents(self.main)
        cmd.redo()
        self.undoStack.clear()

    # delete server component action
    # \brief It deletes the current component from the configuration server
    def serverDeleteComponent(self):
        cmd = ServerDeleteComponent(self.main)
        cmd.redo()
        self.main.componentList.setItemFocus()

    # set component mandatory action
    # \brief It sets the current component as mandatory
    def serverSetMandatoryComponent(self):
        cmd = ServerSetMandatoryComponent(self.main)
        cmd.redo()

    # get mandatory components action
    # \brief It fetches mandatory components
    def serverGetMandatoryComponents(self):
        cmd = ServerGetMandatoryComponents(self.main)
        cmd.redo()

    # unset component mandatory action
    # \brief It unsets the current component as mandatory
    def serverUnsetMandatoryComponent(self):
        cmd = ServerUnsetMandatoryComponent(self.main)
        cmd.redo()

    # fetch server datasources action
    # \brief It fetches datasources from the configuration server
    def serverFetchDataSources(self):
        cmd = ServerFetchDataSources(self.main)
        cmd.redo()
        self.undoStack.clear()

    # store server datasource action
    # \brief It stores the current datasource in the configuration server
    def serverStoreDataSource(self, focus=True):
        cmd = DataSourceEdit(self.main)
        cmd.redo()
        cmd = DataSourceApply(self.main)
        self.undoStack.push(cmd)
        cmd = ServerStoreDataSource(self.main)
        cmd.redo()
        if focus:
            self.main.sourceList.setItemFocus()

    # store server datasource action executed by button
    # \brief It stores the current datasource in
    #        the configuration server executed by button
    def serverStoreDataSourceButton(self):
        if self.main.updateDataSourceListItem():
            self.serverStoreDataSource(False)

    # store server all datasources action
    # \brief It stores all components in the configuration server
    def serverStoreAllDataSources(self):
        cmd = ServerStoreAllDataSources(self.main)
        cmd.redo()
        self.undoStack.clear()

    # delete server datasource action
    # \brief It deletes the current datasource from the configuration server
    def serverDeleteDataSource(self):
        cmd = DataSourceEdit(self.main)
        cmd.redo()
        cmd = ServerDeleteDataSource(self.main)
        cmd.redo()
        self.main.sourceList.setItemFocus()

    # close server action
    # \brief It closes the configuration server
    def serverClose(self):
        cmd = ServerClose(self.main)
        self.undoStack.push(cmd)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file Loggers.py
# Logger classes

""" "Component Desigener Loggers  """

import logging
import sys

from PyQt5.QtCore import (QObject, pyqtSignal)

logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


class LogHandler(logging.Handler):

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):
        message = self.format(record)
        if message:
            LogStream.stdout().write("%s\n" % message)


class LogActions(QObject):
    levels = {'debug': logging.DEBUG,
              'info': logging.INFO,
              'warning': logging.WARNING,
              'error': logging.ERROR,
              'critical': logging.CRITICAL}

    def __init__(self, receiver):
        self.receiver = receiver

    def updatelevel(self):
        levellist = ['no', 'debug', 'info', 'warning', 'error', 'critical']
        eflevel = logger.getEffectiveLevel()
        ilv = eflevel // 10
        level = levellist[ilv]
        self.setlevel(level, True)

    def setlevel(self, level, flag):
        level = level if flag else None
        logger.setLevel(
            self.levels.get(level, logging.NOTSET))
        for lv in self.levels:
            name = "action" + lv[0].upper() + lv[1:]
            if lv != level or not flag:
                if hasattr(self.receiver.ui, name):
                    action = getattr(self.receiver.ui, name)
                    action.setChecked(False)
            else:
                if hasattr(self.receiver.ui, name):
                    action = getattr(self.receiver.ui, name)
                    action.setChecked(True)
        if level in self.levels and flag:
            logger.info("set log level: %s" % level)


class LogStream(QObject):

    written = pyqtSignal(str)
    _stdout = None

    @staticmethod
    def stdout():
        if not LogStream._stdout:
            LogStream._stdout = LogStream()
        return LogStream._stdout

    def write(self, message):
        if not self.signalsBlocked():
            self.written.emit(unicode(message))

    def flush(self):
        pass

    def fileno(self):
        return -1



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file WindowsSlots.py
# user pool commands of GUI application

""" Windows slots """
import sys

from PyQt5.QtGui import QKeySequence
from PyQt5.QtCore import (QSignalMapper, Qt)

from .DataSourceDlg import CommonDataSourceDlg
from .ComponentDlg import ComponentDlg


if sys.version_info > (3,):
    unicode = str


# stack with the application commands
class WindowsSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # dictionary with window actions
        self.windows = {}

        # action data
        self.actions = {
            "actionNextWindows": [
                "&Next", "activateNextSubWindow",
                QKeySequence.NextChild, "", "Go to the next window"],
            "actionPreviousWindows": [
                "&Previous", "activatePreviousSubWindow",
                QKeySequence.PreviousChild, "", "Go to the previous window"],
            "actionCascadeWindows": [
                "Casca&de", "cascadeSubWindows", "", "",
                "Cascade the windows"],
            "actionTileWindows": [
                "&Tile", "tileSubWindows", "", "",
                "Tile the windows"],
            "actionRestoreAllWindows": [
                "&Restore All", "windowRestoreAll", "", "",
                "Restore the windows"],
            "actionCloseAllWindows": [
                "&Close All", "closeAllSubWindows", "", "",
                "Close the windows"],
            "actionIconizeAllWindows": [
                "&Iconize All", "windowMinimizeAll", "", "",
                "Minimize the windows"],
            "actionCloseWindows": [
                "&Close", "closeActiveSubWindow", "",
                QKeySequence(Qt.Key_Escape),
                "Close the window"],
            "actionComponentListWindows": [
                "&Component List", "gotoComponentList", "Ctrl+<", "",
                "Go to the component list"],
            "actionDataSourceListWindows": [
                "&DataSource List", "gotoDataSourceList", "Ctrl+>", "",
                "Go to the component list"]
        }

        for ac in self.actions.keys():
            self.windows[ac] = getattr(self.main.ui, ac)

        self.windows["Mapper"] = QSignalMapper(self.main)
        self.windows["Menu"] = self.main.ui.menuWindow

        # self.main.connect(self.windows["Mapper"], SIGNAL("mapped(QWidget*)"),
        #                   self.main.setActiveSubWindow)
        self.windows["Mapper"].mapped.connect(self.main.setActiveSubWindow)

        # self.main.connect(self.windows["Menu"], SIGNAL("aboutToShow()"),
        #                   self.updateWindowMenu)
        self.windows["Menu"].aboutToShow.connect(self.updateWindowMenu)
        # self.main.connect(
        #     self.main.ui.mdi, SIGNAL("subWindowActivated(QMdiSubWindow*)"),
        #     self.mdiWindowActivated)
        self.main.ui.mdi.subWindowActivated.connect(self.mdiWindowActivated)

    # activated window action, i.e. it changes the current position
    #  of the component and datasource lists
    # \param subwindow selected subwindow
    def mdiWindowActivated(self, subwindow):
        widget = subwindow.widget() if hasattr(subwindow, "widget") else None
        if isinstance(widget, CommonDataSourceDlg):
            if widget.datasource.id is not None:
                if hasattr(self.main.sourceList.currentListElement(), "id"):
                    if self.main.sourceList.currentListElement().id \
                            != widget.datasource.id:
                        self.main.sourceList.populateElements(
                            widget.datasource.id)
        elif isinstance(widget, ComponentDlg):
            if widget.component.id is not None:
                if hasattr(self.main.componentList.currentListElement(), "id"):
                    if self.main.componentList.currentListElement().id \
                            != widget.component.id:
                        self.main.componentList.populateElements(
                            widget.component.id)

    # restores all windows
    # \brief It restores all windows in MDI
    def windowRestoreAll(self):
        for dialog in self.main.ui.mdi.subWindowList():
            dialog.showNormal()

    # minimizes all windows
    # \brief It minimizes all windows in MDI
    def windowMinimizeAll(self):
        for dialog in self.main.ui.mdi.subWindowList():
            dialog.showMinimized()

    # restores all windows
    # \brief It restores all windows in MDI
    def gotoComponentList(self):
        self.main.componentList.ui.elementListWidget.setFocus()

    # restores all windows
    # \brief It restores all windows in MDI
    def gotoDataSourceList(self):
        self.main.sourceList.ui.elementListWidget.setFocus()

    # activates the next subwindow
    def activateNextSubWindow(self):
        self.main.ui.mdi.activateNextSubWindow()

    # activates the previous subwindow
    def activatePreviousSubWindow(self):
        self.main.ui.mdi.activatePreviousSubWindow()

    # cascades the subwindows
    def cascadeSubWindows(self):
        self.main.ui.mdi.cascadeSubWindows()

    # tiles the subwindows
    def tileSubWindows(self):
        self.main.ui.mdi.tileSubWindows()

    # closes all subwindows
    def closeAllSubWindows(self):
        self.main.ui.mdi.closeAllSubWindows()

    # closes the active subwindow
    def closeActiveSubWindow(self):
        self.main.ui.mdi.closeActiveSubWindow()

    # updates the window menu
    # \brief It updates the window menu with the open windows
    def updateWindowMenu(self):
        self.windows["Menu"].clear()
        self._addActions(self.windows["Menu"],
                         (self.windows["actionNextWindows"],
                          self.windows["actionPreviousWindows"],
                          self.windows["actionCascadeWindows"],
                          self.windows["actionTileWindows"],
                          self.windows["actionRestoreAllWindows"],
                          self.windows["actionIconizeAllWindows"],
                          None,
                          self.windows["actionCloseWindows"],
                          self.windows["actionCloseAllWindows"],
                          None,
                          self.windows["actionComponentListWindows"],
                          self.windows["actionDataSourceListWindows"]
                          ))
        dialogs = self.main.ui.mdi.subWindowList()
        if not dialogs:
            return
        self.windows["Menu"].addSeparator()
        i = 1
        menu = self.windows["Menu"]
        for dialog in dialogs:
            title = dialog.windowTitle()
            if i == 10:
                self.windows["Menu"].addSeparator()
                menu = menu.addMenu("&More")
            accel = ""
            if i < 10:
                accel = "&%s " % unicode(i)
            elif i < 36:
                accel = "&%s " % unicode(chr(i + ord("@") - 9))
            action = menu.addAction("%s%s" % (accel, title))

            # self.main.connect(action, SIGNAL("triggered()"),
            #                   self.windows["Mapper"], SLOT("map()"))
            action.triggered.connect(self.windows["Mapper"].map)
            self.windows["Mapper"].setMapping(action, dialog)
            i += 1

    # adds actions to target
    # \param target action target
    # \param actions actions to be added
    @classmethod
    def _addActions(cls, target, actions):
        if not hasattr(actions, '__iter__'):
            target.addAction(actions)
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    # closes the current window
    # \brief Is closes the current datasource window
    def dsourceClose(self):
        subwindow = self.main.ui.mdi.activeSubWindow()
        if subwindow and isinstance(subwindow.widget(), CommonDataSourceDlg) \
                and subwindow.widget().datasource:

            ds = subwindow.widget().datasource

            ds.updateForm()
            if ds.dialog:
                ds.dialog.reject()

            self.main.setActiveSubWindow(subwindow)
            self.main.ui.mdi.closeActiveSubWindow()

    # closes the current window
    # \brief Is closes the current component window
    def componentClose(self):
        subwindow = self.main.ui.mdi.activeSubWindow()
        if subwindow and isinstance(subwindow.widget(), ComponentDlg) \
                and subwindow.widget().component:
            cp = subwindow.widget().component

            if cp.dialog:
                cp.dialog.reject()

            self.main.setActiveSubWindow(subwindow)
            self.main.ui.mdi.closeActiveSubWindow()


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ListSlots.py
# user pool commands of GUI application

""" List slots """

from PyQt5.QtGui import QKeySequence

from .ListCommands import (
    ComponentNew,
    ComponentRemove,
    ComponentListChanged,
    DataSourceNew,
    DataSourceRemove,
    DataSourceListChanged,
    CloseApplication
)

from .EditCommands import (
    ComponentEdit,
    DataSourceEdit
)


# stack with the application commands
class ListSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # action data
        self.actions = {
            "actionClose": [
                "&Remove", "componentRemove",
                "Ctrl+P", "componentremove", "Close the component"],
            "actionCloseDataSource": [
                "&Remove DataSource", "dsourceRemove",
                "Ctrl+Shift+P", "dsourceremove",
                "Close the data source"],
            "actionNew": [
                "&New", "componentNew",
                QKeySequence.New, "componentnew", "Create a new component"],
            "actionNewDataSource": [
                "&New DataSource", "dsourceNew",
                "Ctrl+Shift+N", "dsourceadd",
                "Create a new data source"],
            "actionQuit": [
                "&Quit", "closeApp",
                "Ctrl+Q", "filequit", "Close the application"],

        }

        # task data
        self.tasks = [
            ["dsourceChanged",
             self.main.sourceList.ui.elementListWidget,
             "itemChanged"],
            ["componentChanged",
             self.main.componentList.ui.elementListWidget,
             "itemChanged"],
            ["componentRowChanged",
             self.main.componentList.ui.elementListWidget,
             "currentRowChanged"],
            ["dsourceRowChanged",
             self.main.sourceList.ui.elementListWidget,
             "currentRowChanged"]
        ]

    # remove component action
    # \brief It removes from the component list the current component
    def componentRemove(self):
        cmd = ComponentRemove(self.main)
        self.undoStack.push(cmd)

    # remove datasource action
    # \brief It removes the current datasource
    def dsourceRemove(self):
        cmd = DataSourceRemove(self.main)
        self.undoStack.push(cmd)

    # new component action
    # \brief It creates a new component
    def componentNew(self):
        cmd = ComponentNew(self.main)
        self.undoStack.push(cmd)

    # new datasource action
    # \brief It creates a new datasource
    def dsourceNew(self):
        cmd = DataSourceNew(self.main)
        self.undoStack.push(cmd)

    # close application action
    # \brief It closes the main application
    def closeApp(self):
        cmd = CloseApplication(self.main)
        self.undoStack.push(cmd)

    # component change action
    # \param item new selected item on the component list
    def componentChanged(self, item):
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentListChanged(self.main)
        cmd.item = item
        self.undoStack.push(cmd)

    # datasource change action
    # \param item new selected item ond the datasource list
    def dsourceChanged(self, item):
        cmd = DataSourceEdit(self.main)
        cmd.redo()
        cmd = DataSourceListChanged(self.main)
        cmd.item = item
        self.undoStack.push(cmd)

    # component row change action
    # \param row row Changed
    def componentRowChanged(self, row):
        self.main.deselectComponentSubWindow()

        did = self.main.componentList.currentListElement()
        if did and hasattr(did, 'id'):
            self.main.componentContent(did.id)

    # dsource row change action
    # \param row row Changed
    def dsourceRowChanged(self, row):
        self.main.deselectDataSourceSubWindow()
        did = self.main.sourceList.currentListElement()
        if did and hasattr(did, 'id'):
            self.main.dataSourceContent(did.id)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ListCommands.py
# user commands of GUI application

""" Component Designer commands """

from PyQt5.QtWidgets import (QMessageBox, QUndoCommand)

from .Component import Component
from .LabeledObject import LabeledObject

import sys
import logging
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


# Command which creates a new component
class ComponentNew(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._comp = None

    # executes the command
    # \brief It creates a new component
    def redo(self):

        if self._comp is None:
            self._comp = LabeledObject("", None)
        else:
            self._comp.instance = None

        self.receiver.componentList.addElement(self._comp)
        logger.debug("EXEC componentNew")

    # unexecutes the command
    # \brief It removes the new component
    def undo(self):
        if self._comp is not None:
            self.receiver.componentList.removeElement(self._comp, False)

            if hasattr(self._comp, 'instance') and self._comp.instance:
                subwindow = self.receiver.subWindow(
                    self._comp.instance,
                    self.receiver.ui.mdi.subWindowList())
                if subwindow:
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        logger.debug("UNDO componentNew")


# Command which removes the current component from the component list
class ComponentRemove(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None
        self._subwindow = None

    # executes the command
    # \brief It removes the current component from the component list
    def redo(self):
        if self._cp is not None:
            self.receiver.componentList.removeElement(self._cp, False)
        else:
            self._cp = self.receiver.componentList.currentListElement()
            if self._cp is None:
                QMessageBox.warning(
                    self.receiver,
                    "Component not selected",
                    "Please select one of the components")
            else:
                self.receiver.componentList.removeElement(self._cp, True)

        if hasattr(self._cp, "instance"):
            subwindow = self.receiver.subWindow(
                self._cp.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self.receiver.ui.mdi.closeActiveSubWindow()

        logger.debug("EXEC componentRemove")

    # unexecutes the command
    # \brief It reloads the removed component from the component list
    def undo(self):
        if self._cp is not None:

            self.receiver.componentList.addElement(self._cp, False)
            if self._cp.instance is None:
                self._cp.instance = Component(self.receiver.componentList)
                self._cp.instance.id = self._cp.id
                self._cp.instance.directory = \
                    self.receiver.componentList.directory
                self._cp.instance.name = \
                    self.receiver.componentList.elements[
                        self._cp.id].name

            self._cp.instance.createGUI()
            self._cp.instance.addContextMenu(
                self.receiver.contextMenuActions)

            subwindow = self.receiver.subWindow(
                self._cp.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._cp.instance.dialog.setSaveFocus()
            else:
                if not self._cp.instance.dialog:
                    self._cp.instance.createGUI()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._cp.instance.dialog)
                self._subwindow.resize(680, 560)
                self._cp.instance.dialog.setSaveFocus()
                self._cp.instance.dialog.show()

            self._cp.instance.dialog.show()

            if hasattr(self._cp.instance, "connectExternalActions"):
                self._cp.instance.connectExternalActions(
                    **self.receiver.externalCPActions)

        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()
        logger.debug("UNDO componentRemove")


# Command which changes the current component in the list
class ComponentListChanged(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        # new item
        self.item = None
        # new directory
        self.directory = None
        # new component name
        self.name = None

        self._cp = None
        self._oldName = None
        self._oldDirectory = None

    # executes the command
    # \brief It changes the current component in the list
    def redo(self):
        if self.item is not None or self.name is not None:
            if self.name is None:
                self.name = unicode(self.item.text())
            if self._cp is None:
                self._cp, self._oldName = \
                    self.receiver.componentList.listItemChanged(
                        self.item, self.name)
            else:
                self._cp.name = self.name

            self.receiver.componentList.populateElements(
                self._cp.id if hasattr(self._cp, "id") else None)
            if hasattr(self._cp, "instance") and self._cp.instance is not None:
                self._oldDirectory = self._cp.instance.directory
                self._cp.instance.setName(self.name, self.directory)
            else:
                self._oldDirectory = \
                    self.receiver.componentList.directory

        cp = self.receiver.componentList.currentListElement()
        if hasattr(cp, "id"):
            self.receiver.componentList.populateElements(cp.id)
        else:
            self.receiver.componentList.populateElements()

        logger.debug("EXEC componentChanged")

    # unexecutes the command
    # \brief It changes back the current component in the list
    def undo(self):
        if self._cp is not None:
            self._cp.name = self._oldName
            self.receiver.componentList.addElement(self._cp, False)
            if self._cp.instance is not None:
                self._cp.instance.setName(self._oldName, self._oldDirectory)

        cp = self.receiver.componentList.currentListElement()
        if hasattr(cp, "id"):
            self.receiver.componentList.populateElements(cp.id)
        else:
            self.receiver.componentList.populateElements()

        logger.debug("UNDO componentChanged")


# Command which creates a new datasource
class DataSourceNew(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None

    # executes the command
    # \brief It creates a new datasource
    def redo(self):
        if self._ds is None:
            self._ds = LabeledObject("", None)
        else:
            self._ds.instance = None
        self.receiver.sourceList.addElement(self._ds)
        logger.debug("EXEC dsourceNew")

    # unexecutes the command
    # \brief It removes the added datasource
    def undo(self):
        if self._ds is not None:
            self.receiver.sourceList.removeElement(self._ds, False)

            if hasattr(self._ds, 'instance'):
                subwindow = \
                    self.receiver.subWindow(
                        self._ds.instance,
                        self.receiver.ui.mdi.subWindowList())
                if subwindow:
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        logger.debug("UNDO dsourceNew")


# Command which removes the current datasource from the datasource list
class DataSourceRemove(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None
        self._subwindow = None

    # executes the command
    # \brief It removes the current datasource from the datasource list
    def redo(self):

        if self._ds is not None:
            self.receiver.sourceList.removeElement(self._ds, False)
        else:
            self._ds = self.receiver.sourceList.currentListElement()
            if self._ds is None:
                QMessageBox.warning(
                    self.receiver, "DataSource not selected",
                    "Please select one of the datasources")
            else:
                self.receiver.sourceList.removeElement(self._ds, True)

        if hasattr(self._ds, "instance"):
            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self.receiver.ui.mdi.closeActiveSubWindow()

        logger.debug("EXEC dsourceRemove")

    # unexecutes the command
    # \brief It adds the removes datasource into the datasource list
    def undo(self):
        if self._ds is not None:

            self.receiver.sourceList.addElement(self._ds, False)

            self._ds.instance.createGUI()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.dialog.setSaveFocus()
            else:
                self._ds.instance.createDialog()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(680, 560)
                self._ds.instance.dialog.setSaveFocus()
                self._ds.instance.dialog.show()

            self._ds.instance.dialog.show()
        logger.debug("UNDO dsourceRemove")


# Command which performs change of  the current datasource
class DataSourceListChanged(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        # new item
        self.item = None
        # new datasource name
        self.name = None
        # new datasource directory
        self.directory = None

        self._ds = None
        self._oldName = None
        self._oldDirectory = None

    # executes the command
    # \brief It performs change of  the current datasource
    def redo(self):
        if self.item is not None or self.name is not None:
            if self.name is None:
                self.name = unicode(self.item.text())
            if self._ds is None:
                self._ds, self._oldName = \
                    self.receiver.sourceList.listItemChanged(
                        self.item, self.name)
            else:
                self._ds.name = self.name

            self.receiver.sourceList.populateElements(
                self._ds.id if hasattr(self._ds, "id") else None)
            if hasattr(self._ds, "instance") and self._ds.instance is not None:
                self._oldDirectory = self._ds.instance.directory
                self._ds.instance.setName(self.name, self.directory)
            else:
                self._oldDirectory = self.receiver.sourceList.directory

        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("EXEC dsourceChanged")

    # unexecutes the command
    # \brief It changes back the current datasource
    def undo(self):
        if self._ds is not None:
            self._ds.name = self._oldName
            self.receiver.sourceList.addElement(self._ds, False)
            if self._ds.instance is not None:
                self._ds.instance.setName(
                    self._oldName, self._oldDirectory)

        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("UNDO dsourceChanged")


# Command which is performed during closing the Component Designer
class CloseApplication(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It is performed during closing the Component Designer
    def redo(self):
        if hasattr(self.receiver.ui, 'mdi'):
            self.receiver.close()
            logger.debug("EXEC closeApp")


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ConnectDlg.py
# Connect dialog class

""" server connect widget """

import os
import sys

from PyQt5.QtWidgets import (QDialog, QMessageBox)
from PyQt5 import uic

# from .ui.ui_connectdlg import Ui_ConnectDlg

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "connectdlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a tag connect
class ConnectDlg(QDialog):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(ConnectDlg, self).__init__(parent)

        # device name of the configuration server
        self.device = u''
        # host name of the configuration server
        self.host = u''
        # port of the configuration server
        self.port = None
        # user interface
        self.ui = _formclass()

    # creates GUI
    # \brief It updates GUI and creates connection for required actions
    def createGUI(self):
        self.ui.setupUi(self)
        self.updateForm()
        self.__updateUi()

        self.ui.connectPushButton.clicked.connect(
            self.accept)
        self.ui.cancelPushButton.clicked.connect(self.reject)
        self.ui.deviceLineEdit.textEdited[str].connect(self.__updateUi)

    # updates the connect dialog
    # \brief It sets initial values of the connection form
    def updateForm(self):
        if self.device is not None:
            self.ui.deviceLineEdit.setText(self.device)
        if self.host is not None:
            self.ui.hostLineEdit.setText(self.host)
        if self.port is not None:
            self.ui.portLineEdit.setText(str(self.port))

    # updates connect user interface
    # \brief It sets enable or disable the OK button
    def __updateUi(self):
        enable = bool(self.ui.deviceLineEdit.text())
        self.ui.connectPushButton.setEnabled(enable)

    # accepts input text strings
    # \brief It copies the connect name and value from lineEdit
    #        widgets and accept the dialog
    def accept(self):
        device = unicode(self.ui.deviceLineEdit.text()).strip()
        if not device:
            QMessageBox.warning(self, "Empty device name",
                                "Please define the device name")
            self.ui.deviceLineEdit.setFocus()
            return

        self.device = device
        self.host = unicode(self.ui.hostLineEdit.text()).strip()

        self.port = None
        try:
            port = str(self.ui.portLineEdit.text())
            if port:
                self.port = int(port)
        except Exception:
            QMessageBox.warning(self, "Wrong port number",
                                "Please define the port number")
            self.ui.portLineEdit.setFocus()
            return

        if self.port is not None and not self.host:
            QMessageBox.warning(self, "Empty host name",
                                "Please define the host name")
            self.ui.hostLineEdit.setFocus()
            return

        if self.port is None and self.host:
            QMessageBox.warning(self, "Empty port",
                                "Please define the port")
            self.ui.portLineEdit.setFocus()
            return
        QDialog.accept(self)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # connect form
    form = ConnectDlg()
    form.createGUI()
    form.show()
    app.exec_()

    if form.result():
        if form.device:
            logger.info("Connect: %s , %s , %s" %
                        (form.device, form.host, form.port))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ServerCommands.py
# user commands of GUI application

""" Component Designer commands """

from PyQt5.QtWidgets import (QMessageBox, QUndoCommand, QProgressDialog)

from PyQt5.QtCore import (Qt)

from .DataSourceDlg import (CommonDataSourceDlg)
from . import DataSource
from .ComponentDlg import ComponentDlg
from .Component import Component
from .ComponentCreator import (
    StdComponentCreator, ComponentCreator, DataSourceCreator)
from .LabeledObject import LabeledObject

import logging
import sys
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


# Command which performs connection to the configuration server
class ServerConnect(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._oldstate = None
        self._state = None

    # executes the command
    # \brief It perform connection to the configuration server
    def redo(self):
        if self.receiver.configServer:
            try:
                if self._oldstate is None:
                    self._oldstate = self.receiver.configServer.getState()
                if self._state is None:
                    self.receiver.configServer.open()
                    self._state = self.receiver.configServer.getState()
                else:
                    self.receiver.configServer.setState(self._state)
                    self.receiver.configServer.connect()
                if self.receiver.configServer.connected:
                    self.receiver.disableServer(False)
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in connecting to Configuration Server",
                    unicode(e))

        logger.debug("EXEC serverConnect")

    # unexecutes the command
    # \brief It undo connection to the configuration server,
    #        i.e. it close the connection to the server
    def undo(self):
        if self.receiver.configServer:
            try:
                self.receiver.configServer.close()
                if self._oldstate is None:
                    self.receiver.configServer.setState(self._oldstate)
                self.receiver.disableServer(True)
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in Closing Configuration Server Connection",
                    unicode(e))

        logger.debug("UNDO serverConnect")


# Command which performs connection to the configuration server
class ServerCPCreate(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self.parent = parent
#        self._oldstate = None
#        self._state = None

    # executes the command
    # \brief It perform connection to the configuration server
    def redo(self):
        if self.receiver.configServer:
            if self.receiver.configServer.connected:
                try:
                    cc = ComponentCreator(
                        self.receiver.configServer, self.receiver)
                    if cc.checkOnlineFile(self.receiver.onlineFile):
                        self.receiver.onlineFile = cc.onlineFile
                        cc.create()
                        if cc.action:
                            dsid = None
                            cpid = None
                            for ds, xml in cc.datasources.items():
                                dsid = self.__addDataSource(ds, xml, cc.action)
                            for cp, xml in cc.components.items():
                                cpid = self.__addComponent(cp, xml, cc.action)
                            if cpid:
                                self.receiver.componentList.populateElements(
                                    cpid)
                            if dsid:
                                self.receiver.sourceList.populateElements(dsid)

                except Exception as e:
                    QMessageBox.warning(
                        self.receiver,
                        "Error in creating Component",
                        unicode(e))
        logger.debug("EXEC serverCPCreate")

    def __addComponent(self, name, xml, action):
        cp = LabeledObject(name, None)
        cpEdit = Component(self.receiver.componentList)
        cpEdit.id = cp.id
        cpEdit.directory = self.receiver.componentList.directory
        cpEdit.createGUI()

        cpEdit.name = name
        cpEdit.set(str(xml), True)
        cpEdit.savedXML = None
        cp.name = cpEdit.name
        cp.instance = cpEdit
        self.receiver.componentList.addElement(cp, False)
        if hasattr(cpEdit, "connectExternalActions"):
            cpEdit.connectExternalActions(
                **self.receiver.externalCPActions)
        cpEdit.dialog.setWindowTitle(
            "%s [Component]" % cp.name)

        if action == "STORE":
            self.receiver.configServer.storeComponent(name, xml)
            cpEdit.savedXML = cp.instance.get()
            cp.savedName = cp.name
        elif action == "SAVE":
            cp.instance.merge(False)
            if cp.instance.save():
                cp.savedName = cp.name
                cpEdit.savedXML = cp.instance.get()

        if hasattr(self.receiver.ui, 'mdi'):
            subwindow = self.receiver.subWindow(
                cp.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                cp.instance.dialog.setSaveFocus()
            else:
                subwindow = self.receiver.ui.mdi.addSubWindow(
                    cpEdit.dialog)
                subwindow.resize(680, 560)
                cpEdit.dialog.setSaveFocus()
                cpEdit.dialog.show()
                cp.instance = cpEdit
            cpEdit.dialog.show()
        return cp.id

    def __addDataSource(self, name, xml, action):
        ds = LabeledObject(name, None)
        dsEdit = DataSource.DataSource(self.receiver.sourceList)
        dsEdit.id = ds.id
        dsEdit.directory = self.receiver.sourceList.directory
        dsEdit.name = name

        dsEdit.set(str(xml), True)
        if hasattr(dsEdit, "connectExternalActions"):
            dsEdit.connectExternalActions(
                **self.receiver.externalDSActions)
        ds.name = dsEdit.name
        ds.instance = dsEdit
        self.receiver.sourceList.addElement(ds, False)
        dsEdit.dialog.setWindowTitle(
            "%s [DataSource]" % ds.name)

        if action == "STORE":
            self.receiver.configServer.storeDataSource(name, xml)
            ds.instance.savedXML = ds.instance.get()
            ds.savedName = ds.name
        elif action == "SAVE":
            if ds.instance.save():
                ds.savedName = ds.name
                ds.instance.savedXML = ds.instance.get()

        if hasattr(self.receiver.ui, 'mdi'):
            subwindow = self.receiver.subWindow(
                ds.instance,
                self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                ds.instance.dialog.setSaveFocus()
            else:
                subwindow = self.receiver.ui.mdi.addSubWindow(
                    dsEdit.dialog)
                subwindow.resize(440, 550)
                dsEdit.dialog.setSaveFocus()
                dsEdit.dialog.show()
            dsEdit.dialog.show()
        return ds.id

    # unexecutes the command
    # \brief It undo connection to the configuration server,
    #        i.e. it close the connection to the server
    def undo(self):
        logger.debug("UNDO serverCPCreate")


# Command which performs connection to the configuration server
class ServerSCPCreate(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self.parent = parent
#        self._oldstate = None
#        self._state = None

    # executes the command
    # \brief It perform connection to the configuration server
    def redo(self):
        if self.receiver.configServer:
            if self.receiver.configServer.connected:
                try:
                    cc = StdComponentCreator(
                        self.receiver.configServer, self.receiver)
                    cc.create()
                    if cc.action:
                        dsid = None
                        cpid = None
                        for ds, xml in cc.datasources.items():
                            dsid = self.__addDataSource(ds, xml, cc.action)
                        for cp, xml in cc.components.items():
                            cpid = self.__addComponent(cp, xml, cc.action)
                        if cpid:
                            self.receiver.componentList.populateElements(cpid)
                        if dsid:
                            self.receiver.sourceList.populateElements(dsid)

                except Exception as e:
                    QMessageBox.warning(
                        self.receiver,
                        "Error in creating Standard Component",
                        unicode(e))
        logger.debug("EXEC serverSCPCreate")

    def __addComponent(self, name, xml, action):
        cp = LabeledObject(name, None)
        cpEdit = Component(self.receiver.componentList)
        cpEdit.id = cp.id
        cpEdit.directory = self.receiver.componentList.directory
        cpEdit.createGUI()

        cpEdit.name = name
        cpEdit.set(str(xml), True)
        cpEdit.savedXML = None
        cp.name = cpEdit.name
        cp.instance = cpEdit
        self.receiver.componentList.addElement(cp, False)
        if hasattr(cpEdit, "connectExternalActions"):
            cpEdit.connectExternalActions(
                **self.receiver.externalCPActions)
        cpEdit.dialog.setWindowTitle(
            "%s [Component]" % cp.name)

        if action == "STORE":
            self.receiver.configServer.storeComponent(name, xml)
            cpEdit.savedXML = cp.instance.get()
            cp.savedName = cp.name
        elif action == "SAVE":
            cp.instance.merge(False)
            if cp.instance.save():
                cp.savedName = cp.name
                cpEdit.savedXML = cp.instance.get()

        if hasattr(self.receiver.ui, 'mdi'):
            subwindow = self.receiver.subWindow(
                cp.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                cp.instance.dialog.setSaveFocus()
            else:
                subwindow = self.receiver.ui.mdi.addSubWindow(
                    cpEdit.dialog)
                subwindow.resize(680, 560)
                cpEdit.dialog.setSaveFocus()
                cpEdit.dialog.show()
                cp.instance = cpEdit
            cpEdit.dialog.show()
        return cp.id

    def __addDataSource(self, name, xml, action):
        ds = LabeledObject(name, None)
        dsEdit = DataSource.DataSource(self.receiver.sourceList)
        dsEdit.id = ds.id
        dsEdit.directory = self.receiver.sourceList.directory
        dsEdit.name = name

        dsEdit.set(str(xml), True)
        if hasattr(dsEdit, "connectExternalActions"):
            dsEdit.connectExternalActions(
                **self.receiver.externalDSActions)
        ds.name = dsEdit.name
        ds.instance = dsEdit
        self.receiver.sourceList.addElement(ds, False)
        dsEdit.dialog.setWindowTitle(
            "%s [DataSource]" % ds.name)

        if action == "STORE":
            self.receiver.configServer.storeDataSource(name, xml)
            ds.instance.savedXML = ds.instance.get()
            ds.savedName = ds.name
        elif action == "SAVE":
            if ds.instance.save():
                ds.savedName = ds.name
                ds.instance.savedXML = ds.instance.get()

        if hasattr(self.receiver.ui, 'mdi'):
            subwindow = self.receiver.subWindow(
                ds.instance,
                self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                ds.instance.dialog.setSaveFocus()
            else:
                subwindow = self.receiver.ui.mdi.addSubWindow(
                    dsEdit.dialog)
                subwindow.resize(440, 550)
                dsEdit.dialog.setSaveFocus()
                dsEdit.dialog.show()
            dsEdit.dialog.show()
        return ds.id

    # unexecutes the command
    # \brief It undo connection to the configuration server,
    #        i.e. it close the connection to the server
    def undo(self):
        logger.debug("UNDO serverSCPCreate")


# Command which performs connection to the configuration server
class ServerDSCreate(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self.parent = parent
#        self._oldstate = None
#        self._state = None

    # executes the command
    # \brief It perform connection to the configuration server
    def redo(self):
        if self.receiver.configServer:
            if self.receiver.configServer.connected:
                try:
                    cc = DataSourceCreator(
                        self.receiver.configServer, self.receiver)
                    if cc.checkOnlineFile(self.receiver.onlineFile):
                        self.receiver.onlineFile = cc.onlineFile
                        cc.create()
                        if cc.action:
                            keys = cc.datasources.keys()
                            progress = QProgressDialog(
                                "Storing DataSource elements",
                                "", 0, len(keys),
                                self.receiver.sourceList)
                            progress.setWindowTitle(
                                "Store Created DataSources")
                            progress.setWindowModality(Qt.WindowModal)
                            progress.setCancelButton(None)
                            progress.show()

                            i = 0
                            dsid = None
                            for ds, xml in cc.datasources.items():
                                dsid = self.__addDataSource(ds, xml, cc.action)
                                progress.setValue(i)
                                i += 1
                            progress.setValue(len(keys))
                            progress.close()
                            if dsid:
                                self.receiver.sourceList.populateElements(dsid)

                except Exception as e:
                    QMessageBox.warning(
                        self.receiver,
                        "Error in creating Component",
                        unicode(e))
        logger.debug("EXEC serverDSCreate")

    def __addDataSource(self, name, xml, action):
        ds = LabeledObject(name, None)
        dsEdit = DataSource.DataSource(self.receiver.sourceList)
        dsEdit.id = ds.id
        dsEdit.directory = self.receiver.sourceList.directory
        dsEdit.name = name

        dsEdit.set(str(xml), True)
        if hasattr(dsEdit, "connectExternalActions"):
            dsEdit.connectExternalActions(
                **self.receiver.externalDSActions)
        ds.name = dsEdit.name
        ds.instance = dsEdit
        self.receiver.sourceList.addElement(ds, False)
        dsEdit.dialog.setWindowTitle(
            "%s [DataSource]" % ds.name)

        if action == "STORE":
            self.receiver.configServer.storeDataSource(name, xml)
            ds.instance.savedXML = ds.instance.get()
            ds.savedName = ds.name
        elif action == "SAVE":
            if ds.instance.save():
                ds.savedName = ds.name
                ds.instance.savedXML = ds.instance.get()

        if hasattr(self.receiver.ui, 'mdi'):
            subwindow = self.receiver.subWindow(
                ds.instance,
                self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                ds.instance.dialog.setSaveFocus()
            else:
                subwindow = self.receiver.ui.mdi.addSubWindow(
                    dsEdit.dialog)
                subwindow.resize(440, 550)
                dsEdit.dialog.setSaveFocus()
                dsEdit.dialog.show()
            dsEdit.dialog.show()
        return ds.id

    # unexecutes the command
    # \brief It undo connection to the configuration server,
    #        i.e. it close the connection to the server
    def undo(self):
        logger.debug("UNDO serverDSCreate")


# Command which fetches the components from the configuration server
class ServerFetchComponents(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It fetches the components from the configuration server
    def redo(self):
        failures = []
        if not self.receiver.closeList(
                None, self.receiver.componentList, failures):
            return
        if (failures and QMessageBox.question(
                self.receiver,
                "Component - Reload List from Configuration server",
                ("All unsaved components will be lost. "
                 "Would you like to proceed ?").encode(),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes) == QMessageBox.No):
            return

        subwindows = self.receiver.ui.mdi.subWindowList()
        if subwindows:
            for subwindow in subwindows:
                dialog = subwindow.widget()
                if isinstance(dialog, ComponentDlg):
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.componentList.elements = {}

        if self.receiver.configServer:
            try:
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )

                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                cdict = self.receiver.configServer.fetchComponents()
                self.receiver.setComponents(cdict)
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in fetching components", unicode(e))

        logger.debug("EXEC serverFetchComponents")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO serverFetchComponents")


# Command which stores the current component in the configuration server
class ServerStoreComponent(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None
        self._cpEdit = None

    # executes the command
    # \brief It stores the current component in the configuration server
    def redo(self):
        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is not None:
            if self._cp.instance is None:
                #                self._cpEdit = FieldWg()
                self._cpEdit = Component(self.receiver.componentList)
                self._cpEdit.id = self._cp.id
                self._cpEdit.directory = \
                    self.receiver.componentList.directory
                self._cpEdit.name = \
                    self.receiver.componentList.elements[
                        self._cp.id].name
                self._cpEdit.createGUI()
                self._cpEdit.addContextMenu(
                    self.receiver.contextMenuActions)
                self._cpEdit.createHeader()
                self._cpEdit.dialog.setWindowTitle(
                    "%s [Component]" % self._cp.name)
            else:
                self._cpEdit = self._cp.instance

            if hasattr(self._cpEdit, "connectExternalActions"):
                self._cpEdit.connectExternalActions(
                    **self.receiver.externalCPActions)

            subwindow = self.receiver.subWindow(
                self._cpEdit, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
            else:
                self._cpEdit.createGUI()

                self._cpEdit.addContextMenu(
                    self.receiver.contextMenuActions)
                if self._cpEdit.isDirty():
                    self._cpEdit.dialog.setWindowTitle(
                        "%s [Component]*" % self._cp.name)
                else:
                    self._cpEdit.dialog.setWindowTitle(
                        "%s [Component]" % self._cp.name)

                self._cpEdit.reconnectSaveAction()
                subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._cpEdit.dialog)
                subwindow.resize(680, 560)
                self._cpEdit.dialog.show()
                self._cp.instance = self._cpEdit

            try:
                xml = self._cpEdit.get()
                if not self.receiver.configServer:
                    raise Exception("Server not connected")

                if not self.receiver.configServer.connected:
                    if QMessageBox.question(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        ),
                            QMessageBox.Yes | QMessageBox.No,
                            QMessageBox.Yes) == QMessageBox.No:
                        raise Exception("Server not connected")

                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                self.receiver.configServer.storeComponent(
                    self._cpEdit.name, xml)
                self._cpEdit.savedXML = xml
                self._cp.savedName = self._cp.name
            except Exception as e:
                QMessageBox.warning(self.receiver,
                                    "Error in storing the component",
                                    unicode(e))
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()

        logger.debug("EXEC serverStoreComponent")

    # unexecutes the command
    # \brief It populates only the component list
    def undo(self):
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()
        logger.debug("UNDO serverStoreComponent")


# Command which deletes the current component from the configuration server
class ServerDeleteComponent(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None

    # executes the command
    # \brief It deletes the current component from the configuration server
    def redo(self):
        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is not None:

            try:
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )

                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                self.receiver.configServer.deleteComponent(self._cp.name)
                self._cp.savedName = ""
                if hasattr(self._cp, "instance"):
                    self._cp.instance.savedXML = ""
            except Exception as e:
                QMessageBox.warning(
                    self.receiver, "Error in deleting the component",
                    unicode(e))

        cid = self._cp.id if hasattr(self._cp, "id") else None
        self.receiver.componentList.populateElements(cid)

        logger.debug("EXEC serverDeleteComponent")

    # unexecutes the command
    # \brief It populates only the component list
    def undo(self):
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()
        logger.debug("UNDO serverDeleteComponent")


# Command which sets on the configuration server the current component
#  as mandatory
class ServerSetMandatoryComponent(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None

    # executes the command
    # \brief It sets on the configuration server the current component
    #        as mandatory
    def redo(self):
        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is not None:
            try:
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )

                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                self.receiver.configServer.setMandatory(self._cp.name)
                mandatory = self.receiver.configServer.getMandatory()
                logger.info("Mandatory Components: \n %s" % str(mandatory))
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in setting the component as mandatory",
                    unicode(e))
        logger.debug("EXEC serverSetMandatoryComponent")


# Command which fetches a list of the mandatory components from
#  the configuration server
class ServerGetMandatoryComponents(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It fetches a list of the mandatory components from
    #        the configuration server
    def redo(self):
        try:
            if not self.receiver.configServer.connected:
                QMessageBox.information(
                    self.receiver,
                    "Connecting to Configuration Server",
                    "Connecting to %s on %s:%s" % (
                        self.receiver.configServer.device,
                        self.receiver.configServer.host,
                        self.receiver.configServer.port
                    )
                )

            self.receiver.configServer.connect()
            self.receiver.disableServer(False)
            mandatory = self.receiver.configServer.getMandatory()
            logger.info("Mandatory Components: \n %s" % str(mandatory))
            QMessageBox.information(
                self.receiver, "Mandatory",
                "Mandatory Components: \n %s" % unicode(mandatory))

        except Exception as e:
            QMessageBox.warning(
                self.receiver,
                "Error in getting the mandatory components",
                unicode(e))
        logger.debug("EXEC serverGetMandatoryComponent")


# Command which sets on the configuration server the current component
#  as not mandatory
class ServerUnsetMandatoryComponent(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None

    # executes the command
    # \brief It sets on the configuration server the current component
    #        as not mandatory
    def redo(self):
        if self._cp is None:
            self._cp = \
                self.receiver.componentList.currentListElement()
        if self._cp is not None:
            try:
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )

                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                self.receiver.configServer.unsetMandatory(
                    self._cp.name)
                mandatory = self.receiver.configServer.getMandatory()
                logger.info("Mandatory Components: \n %s" % str(mandatory))
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in setting the component as mandatory",
                    unicode(e))
        logger.debug("EXEC serverUnsetMandatoryComponent")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO serverUnsetMandatoryComponent")


# Command which fetches the datasources from the configuration server
class ServerFetchDataSources(QUndoCommand):
    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It fetches the datasources from the configuration server
    def redo(self):
        failures = []
        if not self.receiver.closeList(
                None, self.receiver.sourceList, failures):
            return
        if (failures and QMessageBox.question(
                self.receiver,
                "DataSource - Reload List from Configuration Server",
                ("All unsaved datasources will be lost. "
                 "Would you like to proceed ?").encode(),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes) == QMessageBox.No):
            return

        subwindows = self.receiver.ui.mdi.subWindowList()
        if subwindows:
            for subwindow in subwindows:
                dialog = subwindow.widget()
                if isinstance(dialog, CommonDataSourceDlg):
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.sourceList.elements = {}

        if self.receiver.configServer:
            try:
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )
                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                cdict = self.receiver.configServer.fetchDataSources()
                self.receiver.setDataSources(cdict)
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in fetching datasources", unicode(e))

        logger.debug("EXEC serverFetchDataSources")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO serverFetchDataSources")


# Command which stores the current datasource in the configuration server
class ServerStoreDataSource(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None

    # executes the command
    # \brief It fetches the datasources from the configuration server
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is not None and hasattr(self._ds, "instance"):
            try:
                xml = self._ds.instance.get()
                if not self.receiver.configServer:
                    raise Exception("Server not connected")

                if not self.receiver.configServer.connected:
                    if QMessageBox.question(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        ),
                            QMessageBox.Yes | QMessageBox.No,
                            QMessageBox.Yes) == QMessageBox.No:
                        raise Exception("Server not connected")

                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                if self._ds.instance.name:
                    self.receiver.configServer.storeDataSource(
                        self._ds.instance.dataSourceName, xml)
                else:
                    self.receiver.configServer.storeDataSource(
                        self._ds.instance.name, xml)
                self._ds.instance.savedXML = xml
                self._ds.savedName = self._ds.name
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in datasource storing", unicode(e))

        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("EXEC serverStoreDataSource")

    # unexecutes the command
    # \brief It populates the datasource list
    def undo(self):
        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()
        logger.debug("UNDO serverStoreDataSource")


# Command which deletes the current datasource in the configuration server
class ServerDeleteDataSource(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None

    # executes the command
    # \brief It deletes the current datasource in the configuration server
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is not None:
            try:
                if hasattr(self._ds, "instance"):
                    self._ds.instance.savedXML = ""
                    name = self._ds.instance.dataSourceName
                    if name is None:
                        name = ""
                    if not self.receiver.configServer.connected:
                        QMessageBox.information(
                            self.receiver,
                            "Connecting to Configuration Server",
                            "Connecting to %s on %s:%s" % (
                                self.receiver.configServer.device,
                                self.receiver.configServer.host,
                                self.receiver.configServer.port
                            )
                        )

                    self.receiver.configServer.connect()
                    self.receiver.disableServer(False)
                    self.receiver.configServer.deleteDataSource(name)
                    self._ds.savedName = ""

            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in datasource deleting", unicode(e))

        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()
        logger.debug("EXEC serverDeleteDataSource")

    # unexecutes the command
    # \brief It populates the datasource list
    def undo(self):
        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()
        logger.debug("UNDO serverDeleteDataSource")


# Command which closes connection to the configuration server
class ServerClose(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._state = None

    # executes the command
    # \brief It closes connection to the configuration server
    def redo(self):
        if self.receiver.configServer:
            self.receiver.configServer.close()

        if self.receiver.configServer:
            try:
                if self._state is None:
                    self._state = self.receiver.configServer.getState()
                self.receiver.configServer.close()
                self.receiver.disableServer(True)
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in closing connection to Configuration Server",
                    unicode(e))

        logger.debug("EXEC serverClose")

    # unexecutes the command
    # \brief It reopen the connection to the configuration server
    def undo(self):
        if self.receiver.configServer:
            try:
                if self._state is None:
                    self.receiver.configServer.open()
                else:
                    self.receiver.configServer.setState(self._state)
                    self.receiver.configServer.connect()
                self.receiver.disableServer(False)
            except Exception as e:
                QMessageBox.warning(
                    self.receiver,
                    "Error in connecting to Configuration Server",
                    unicode(e))
        logger.debug("UNDO serverClose")


# Command which saves all components in the file
class ServerStoreAllComponents(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._subwindow = None

    # executes the command
    # \brief It saves all components in the file
    def redo(self):

        keys = self.receiver.componentList.elements.keys()
        progress = QProgressDialog(
            "Storing Component elements",
            "", 0, len(keys), self.receiver.componentList)
        progress.setWindowTitle("Store All Components")
        progress.setWindowModality(Qt.WindowModal)
        progress.setCancelButton(None)
        progress.show()

        for i in range(len(keys)):
            icp = keys[i]
            cp = self.receiver.componentList.elements[icp]
            if cp.instance is None:
                #                self._cpEdit = FieldWg()
                cpEdit = Component(self.receiver.componentList)
                cpEdit.id = cp.id
                cpEdit.directory = self.receiver.componentList.directory
                cpEdit.name = self.receiver.componentList.elements[
                    cp.id].name
                cpEdit.createGUI()
                cpEdit.addContextMenu(self.receiver.contextMenuActions)
                cpEdit.createHeader()
                cpEdit.dialog.setWindowTitle("%s [Component]" % cp.name)
                cp.instance = cpEdit

            try:
                cp.instance.merge(False)
                xml = cp.instance.get()
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )
                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                self.receiver.configServer.storeComponent(
                    cp.instance.name, xml)
                cp.instance.savedXML = xml
                cp.savedName = cp.name
            except Exception as e:
                QMessageBox.warning(
                    self.receiver, "Error in storing the component",
                    unicode(e))
            progress.setValue(i)
        progress.setValue(len(keys))
        progress.close()
        if hasattr(cp, "id"):
            self.receiver.componentList.populateElements(cp.id)
        else:
            self.receiver.componentList.populateElements()

        logger.debug("EXEC componentStoreAll")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO componentStoreAll")


# Command which saves all the datasources in files
class ServerStoreAllDataSources(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It saves all the datasources in files
    def redo(self):

        keys = self.receiver.sourceList.elements.keys()
        progress = QProgressDialog(
            "Storing DataSource elements",
            "", 0, len(keys), self.receiver.sourceList)
        progress.setWindowTitle("Store All DataSources")
        progress.setWindowModality(Qt.WindowModal)
        progress.setCancelButton(None)
        progress.show()

        for i in range(len(keys)):
            ids = keys[i]
            ds = self.receiver.sourceList.elements[ids]
            if ds.instance is None:
                dsEdit = DataSource.DataSource(self.receiver.sourceList)
                dsEdit.id = ds.id
                dsEdit.directory = self.receiver.sourceList.directory
                dsEdit.name = \
                    self.receiver.sourceList.elements[ds.id].name
                ds.instance = dsEdit
            logger.debug("Store %s" % ds.instance.name)

            try:
                xml = ds.instance.get()
                if not self.receiver.configServer.connected:
                    QMessageBox.information(
                        self.receiver,
                        "Connecting to Configuration Server",
                        "Connecting to %s on %s:%s" % (
                            self.receiver.configServer.device,
                            self.receiver.configServer.host,
                            self.receiver.configServer.port
                        )
                    )
                self.receiver.configServer.connect()
                self.receiver.disableServer(False)
                if ds.instance.name:
                    self.receiver.configServer.storeDataSource(
                        ds.instance.dataSourceName, xml)
                else:
                    self.receiver.configServer.storeDataSource(
                        ds.instance.name, xml)
                ds.instance.savedXML = xml
                ds.savedName = ds.name
            except Exception as e:
                QMessageBox.warning(
                    self.receiver, "Error in datasource storing",
                    unicode(e))

            progress.setValue(i)
        progress.setValue(len(keys))
        progress.close()
        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("EXEC dsourceStoreAll")

    # executes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO dsourceStoreAll")


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file StrategyDlg.py
# Strategy dialog class

""" strategy widget """

import os
import sys

from PyQt5.QtCore import (QModelIndex)
from PyQt5 import uic

# from .ui.ui_strategydlg import Ui_StrategyDlg
from .NodeDlg import NodeDlg
from .DomTools import DomTools

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "strategydlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining an attribute
class StrategyDlg(NodeDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(StrategyDlg, self).__init__(parent)

        # strategy mode
        self.mode = u''
        # trigger label
        self.trigger = u''
        # growing dimension
        self.grows = u''
        # postrun  label
        self.postrun = u''
        # attribute doc
        self.doc = u''
        # compression flag
        self.compression = False
        # compression rate
        self.rate = 5
        # compression shuffle
        self.shuffle = True

        # allowed subitems
        self.subItems = ["doc"]

        # writing can fail
        self.canfail = False

        # user interface
        self.ui = _formclass()

    # updates the field strategy
    # \brief It sets the form local variables
    def updateForm(self):
        index = -1
        if self.mode is not None:
            index = self.ui.modeComboBox.findText(unicode(self.mode))
            if index > -1:
                self.ui.modeComboBox.setCurrentIndex(index)
                self.setFrames(self.mode)
        if index < 0 or index is None:
            index2 = self.ui.modeComboBox.findText(unicode("STEP"))
            self.mode = 'STEP'
            self.ui.modeComboBox.setCurrentIndex(index2)
            self.setFrames(self.mode)

        if self.trigger is not None:
            self.ui.triggerLineEdit.setText(self.trigger)

        self.ui.compressionCheckBox.setChecked(self.compression)
        self.ui.canFailCheckBox.setChecked(self.canfail)
        self.ui.shuffleCheckBox.setChecked(self.shuffle)
        self.ui.rateSpinBox.setValue(self.rate)

        if self.grows is not None:
            try:
                grows = int(self.grows)
                if grows < 0:
                    grows = 0
            except Exception:
                grows = 0
            self.ui.growsSpinBox.setValue(grows)
        if self.postrun is not None:
            self.ui.postLineEdit.setText(self.postrun)
        if self.doc is not None:
            self.ui.docTextEdit.setText(self.doc)

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):
        self.ui.setupUi(self)

        self.updateForm()

        self.ui.resetPushButton.clicked.connect(self.reset)
        self.ui.modeComboBox.currentIndexChanged[str].connect(self.setFrames)
        self.ui.compressionCheckBox.stateChanged[int].connect(
            self.setCompression)

        self.setCompression(self.ui.compressionCheckBox.isChecked())

    # provides the state of the strategy dialog
    # \returns state of the strategy in tuple
    def getState(self):
        state = (self.mode,
                 self.trigger,
                 self.grows,
                 self.postrun,
                 self.compression,
                 self.rate,
                 self.shuffle,
                 self.canfail,
                 self.doc
                 )
        return state

    # sets the state of the strategy dialog
    # \param state strategy state written in tuple
    def setState(self, state):

        (self.mode,
         self.trigger,
         self.grows,
         self.postrun,
         self.compression,
         self.rate,
         self.shuffle,
         self.canfail,
         self.doc
         ) = state

    # shows and hides frames according to modeComboBox
    # \param text the edited text
    def setFrames(self, text):
        if text == 'STEP':
            self.ui.triggerFrame.show()
            self.ui.postFrame.hide()
            self.ui.triggerLineEdit.setFocus()
        elif text == 'POSTRUN':
            self.ui.postFrame.show()
            self.ui.triggerFrame.hide()
            self.ui.postLineEdit.setFocus()
        else:
            self.ui.postFrame.hide()
            self.ui.triggerFrame.hide()

    # shows and hides compression widgets according to compressionCheckBox
    # \param state value from compressionCheckBox
    def setCompression(self, state):
        enable = bool(state)
        self.ui.rateLabel.setEnabled(enable)
        self.ui.rateSpinBox.setEnabled(enable)
        self.ui.shuffleCheckBox.setEnabled(enable)

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if node:
            # defined in NodeDlg class
            self.node = node
        if not self.node:
            return
        attributeMap = self.node.attributes()

        self.trigger = attributeMap.namedItem("trigger").nodeValue() \
            if attributeMap.contains("trigger") else ""
        self.grows = attributeMap.namedItem("grows").nodeValue() \
            if attributeMap.contains("grows") else ""
        self.mode = attributeMap.namedItem("mode").nodeValue() \
            if attributeMap.contains("mode") else ""

        if attributeMap.contains("canfail"):
            self.canfail = \
                False if str(attributeMap.namedItem("canfail").nodeValue()
                             ).upper() == "FALSE" else True

        if attributeMap.contains("compression"):
            self.compression = \
                False if str(attributeMap.namedItem("compression").nodeValue()
                             ).upper() == "FALSE" else True

            if attributeMap.contains("shuffle"):
                self.shuffle = \
                    False if str(attributeMap.namedItem("shuffle").nodeValue()
                                 ).upper() == "FALSE" else True

            if attributeMap.contains("rate"):
                rate = int(attributeMap.namedItem("rate").nodeValue())
                if rate < 0:
                    rate = 0
                elif rate > 9:
                    rate = 9
                self.rate = rate

        text = DomTools.getText(self.node)
        self.postrun = unicode(text).strip() if text else ""

        doc = self.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.doc = unicode(text).strip() if text else ""

    # accepts input text strings
    # \brief It copies the attribute name and value from
    #        lineEdit widgets and accept the dialog
    def apply(self):

        self.trigger = ''
        self.grows = ''
        self.postrun = ''

        self.mode = unicode(self.ui.modeComboBox.currentText())
        if self.mode == 'STEP':
            self.trigger = unicode(self.ui.triggerLineEdit.text())
            grows = int(self.ui.growsSpinBox.value())
            if grows > 0:
                self.grows = str(grows)
        if self.mode == 'POSTRUN':
            self.postrun = unicode(self.ui.postLineEdit.text())

        self.canfail = self.ui.canFailCheckBox.isChecked()
        self.compression = self.ui.compressionCheckBox.isChecked()
        self.shuffle = self.ui.shuffleCheckBox.isChecked()
        self.rate = self.ui.rateSpinBox.value()

        self.doc = unicode(self.ui.docTextEdit.toPlainText())

        index = self.view.currentIndex()
        finalIndex = self.view.model().createIndex(
            index.row(), 2, index.parent().internalPointer())

        self.view.expand(index)
        if self.node and self.root and self.node.isElement():
            self.updateNode(index)
        if index.column() != 0:
            index = self.view.model().index(
                index.row(), 0, index.parent())
        self.view.model().dataChanged.emit(index, finalIndex)
        self.view.expand(index)

    # updates the Node
    # \brief It sets node from the dialog variables
    def updateNode(self, index=QModelIndex()):
        elem = self.node.toElement()

        mindex = self.view.currentIndex() if not index.isValid() else index

        attributeMap = self.node.attributes()
        for _ in range(attributeMap.count()):
            attributeMap.removeNamedItem(attributeMap.item(0).nodeName())
        elem.setAttribute(str("mode"), str(self.mode))

        if self.mode == 'STEP':
            if self.trigger:
                elem.setAttribute(str("trigger"), str(self.trigger))
            if self.grows:
                elem.setAttribute(str("grows"), str(str(self.grows)))
        if self.canfail:
            elem.setAttribute(str("canfail"), str("true"))
        if self.compression:
            elem.setAttribute(str("compression"), str("true"))
            elem.setAttribute(str("shuffle"), str("true")
                              if self.shuffle else "false")
            elem.setAttribute(str("rate"), str(str(self.rate)))

        self.replaceText(mindex, unicode(self.postrun))

        doc = self.node.firstChildElement(str("doc"))
        if not self.doc and doc and doc.nodeName() == "doc":
            self.removeElement(doc, mindex)

        elif self.doc:
            newDoc = self.root.createElement(str("doc"))
            newText = self.root.createTextNode(str(self.doc))
            newDoc.appendChild(newText)
            if doc and doc.nodeName() == "doc":
                self.replaceElement(doc, newDoc, mindex)
            else:
                self.appendElement(newDoc, mindex)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # attribute form
    form = StrategyDlg()

    form.mode = "pre_sample_flightpath"
    form.trigger = "trigger1"
    form.postrun = 'http://haso.desy.de:/data/power.dat'
    form.doc = "This is the flightpath before the sample position."
    form.createGUI()
    form.show()
    app.exec_()

    if form.mode:
        logger.info("Mode: %s " % form.mode)
    if form.mode == "STEP":
        if form.trigger:
            logger.info("Trigger: %s" % form.trigger)
    if form.mode == "POSTRUN":
        if form.postrun:
            logger.info("Postrun label: %s" % form.postrun)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file LabeledObject.py
# object with label and ID

""" labeled object for component and datasource lists"""


# item of the component or datasource list
class LabeledObject(object):
    # constructor
    # \param name item name
    # \param instance instance related to the item
    def __init__(self, name, instance):
        # item name
        self.name = name
        # saved item name
        self.savedName = name
        # item instance
        self.instance = instance
        # item id
        self.id = id(self)

    # checks if the name is not saved
    # returns False if the name is not saved
    def isDirty(self):
        return False if self.name == self.savedName else True



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file NodeDlg.py
# Abstract Node dialog class

""" abstract Node widget """

from PyQt5.QtWidgets import QDialog
from PyQt5.QtCore import (QModelIndex)

from .DomTools import DomTools


# abstract node dialog
class NodeDlg(QDialog):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(NodeDlg, self).__init__(parent)

        # DOM node
        self.node = None
        # DOM root
        self.root = None
        # component tree view
        self.view = None

        # allowed subitems
        self.subItems = []

        #  user interface
        self.ui = None

        # external apply action
        self.externalApply = None

        # external apply action
        self.externalDSLink = None

        # datasource label for templated XML
        self.dsLabel = "datasources"

    # connects the save action and stores the apply action
    # \param externalApply apply action
    # \param externalSave save action
    # \param externalClose close action
    # \param externalStore store action
    # \param externalDSLink dsource link action
    def connectExternalActions(self, externalApply=None, externalSave=None,
                               externalClose=None, externalStore=None,
                               externalDSLink=None):
        if externalApply and self.externalApply is None and self.ui and \
                hasattr(self.ui, "applyPushButton") and \
                self.ui.applyPushButton:
            self.ui.applyPushButton.clicked.connect(externalApply)
            self.externalApply = externalApply
        if externalDSLink and self.externalDSLink is None and self.ui and \
                hasattr(self.ui, "linkDSPushButton") \
                and self.ui.linkDSPushButton:
            self.ui.linkDSPushButton.clicked.connect(externalDSLink)
            self.externalDSLink = externalDSLink

    # resets the dialog
    # \brief It sets forms and dialog from DOM
    def reset(self):
        if self.view and hasattr(self.view, "currentIndex"):
            index = self.view.currentIndex()
        self.setFromNode()
        self.updateForm()
        if self.view:
            if index.column() != 0:
                index = self.view.model().index(index.row(), 0, index.parent())
            self.view.model().dataChanged.emit(index, index)

    # replaces node text for the given node
    # \param index of child text node
    # \param text string with text
    def replaceText(self, index, text=None):
        if self.view is not None and self.view.model() is not None:
            return DomTools.replaceText(
                self.node, index, self.view.model(), text)

    # removes node element
    # \param element DOM node element to remove
    # \param parent parent node index
    def removeElement(self, element, parent):
        if self.view is not None and self.view.model() is not None:
            return DomTools.removeElement(
                element, parent, self.view.model())

    # replaces node element
    # \param oldElement old DOM node element
    # \param newElement new DOM node element
    # \param parent parent node index
    def replaceElement(self, oldElement, newElement, parent):
        if self.view is not None and self.view.model() is not None:
            return DomTools.replaceElement(
                oldElement, newElement, parent, self.view.model())

    # appends node element
    # \param newElement new DOM node element
    # \param parent parent node index
    def appendElement(self, newElement, parent):
        if self.view is not None and self.view.model() is not None:
            return DomTools.appendNode(
                newElement, parent, self.view.model())
        return False

    # updates the form
    # \brief abstract class
    def updateForm(self):
        pass

    # updates the node
    # \brief abstract class
    def updateNode(self, index=QModelIndex()):
        pass

    # creates GUI
    # \brief abstract class
    def createGUI(self):
        pass

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        pass


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DataSourceList.py
# Data source list class

""" datasource list widget """


from .DataSource import DataSource
from .ElementList import ElementList

import logging
# message logger
logger = logging.getLogger("nxsdesigner")


# dialog defining a group tag
class DataSourceList(ElementList):

    # constructor
    # \param directory element directory
    # \param parent patent instance
    def __init__(self, directory, parent=None):
        super(DataSourceList, self).__init__(directory, parent)

        # widget title
        self.title = "DataSources"
        # element name
        self.name = "datasources"
        # class name
        self.clName = "DataSource"
        # extention
        self.extention = ".ds.xml"

    # retrives element name from file name
    # \param fname filename
    # \returns element name
    @classmethod
    def nameFromFile(cls, fname):
        if fname[-4:] == '.xml':
            name = fname[:-4]
            if name[-3:] == '.ds':
                name = name[:-3]
        else:
            name = fname
        return name

    # creates Element
    # \param name element name
    # \returns element instance
    def createElement(self, name):
        dlg = DataSource(self)
        dlg.directory = self.directory
        dlg.name = name
        dlg.createGUI()
        return dlg

    # replaces name special characters by underscore
    # \param name give name
    # \returns replaced element
    @classmethod
    def dashName(cls, name):
        res = "".join(
            x.replace('/', '_').replace('\\', '_').replace(':', '_')
            for x in name if (x.isalnum() or x in ["/", "\\", ":", "_"]))
        return res


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # group form
    form = DataSourceList("../datasources")
#    form.elements={"title":"Test run 1", "run_cycle":"2012-1"}
    form.createGUI()
    form.show()
    app.exec_()

    if form.elements:
        logger.info("Other datasources:")
        for k in form.elements.keys():
            logger.info(" %s = '%s' " % (k, form.elements[k]))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ComponentCreator.py
# Class for connecting to configuration server

""" Provides connects to configuration server"""

from PyQt5.QtWidgets import (QMessageBox, QFileDialog)
from .CreatorDlg import CreatorDlg, StdCreatorDlg

import logging
import sys
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str

try:
    import nxstools
    from nxstools.nxscreator import (
        OnlineCPCreator, OnlineDSCreator, StandardCPCreator)
    NXSTOOLS_AVAILABLE = True
    NXSTOOLS_MASTERVER, NXSTOOLS_MINORVER, NXSTOOLS_PATCHVER = \
        nxstools.__version__.split(".")
    if int(NXSTOOLS_MASTERVER) <= 2:
        if int(NXSTOOLS_MINORVER) <= 2:
            raise ImportError("nxstools is below 2.3.0")
except ImportError as e:
    NXSTOOLS_AVAILABLE = False
    logger.info("nxstools is not available: %s" % e)


# option class
class Options(object):
    def __init__(self, server=None):
        self.server = server
        self.overwrite = True
        self.lower = True
        self.component = None
        self.cptype = None
        self.directory = None
        self.xmlpackage = None
        self.external = ""
        self.entryname = "scan"
        self.insname = "instrument"
        self.oldclientlike = False
        self.clientlike = True


# configuration server
class ComponentCreator(object):

    # constructor
    def __init__(self, configServer, parent):
        # configuration server
        self.configServer = configServer

        # online file name
        self.onlineFile = None

        # componentName
        self.componentName = None

        # created components
        self.components = {}
        # created components
        self.datasources = {}
        # available components
        self.avcomponents = []
        # parent
        self.parent = parent

    # sets onlineFile name and check if online file exists
    def checkOnlineFile(self, onlineFile):
        onlineFile = onlineFile or '/online_dir/online.xml'
        onlineFile = unicode(QFileDialog.getOpenFileName(
            self.parent, "Open Online File", onlineFile,
            "XML files (*.xml)")[0])
        if onlineFile:
            self.onlineFile = onlineFile
            return True
        else:
            self.onlineFile = None

    # creates component and datasources from online xml
    def create(self):
        self.action = None
        self.componentName = None
        self.components = {}
        self.datasources = {}
        if NXSTOOLS_AVAILABLE and self.onlineFile:
            options = Options(self.configServer.getDeviceName())

            occ = OnlineCPCreator(options, [self.onlineFile], False)
            self.avcomponents = occ.listcomponents() or []
            if self.avcomponents:
                self.action = self.selectComponent()
                options.component = self.componentName
                if self.action:
                    occ.createXMLs()
                    self.components = dict(occ.components)
                    self.datasources = dict(occ.datasources)
            else:
                QMessageBox.warning(
                    self.parent,
                    "Error in creating Component",
                    "Cannot find any known components in '%s'" %
                    self.onlineFile)

    # runs Creator widget to select the required component
    # \returns action to be performed with the components and datasources
    def selectComponent(self):
        aform = CreatorDlg(self.parent)
        if self.avcomponents:
            aform.components = list(self.avcomponents)

        action = None
        self.componentName = None
        aform.createGUI()
        aform.show()
        if aform.exec_():
            action = aform.action
            self.componentName = aform.componentName
        return action


# configuration server
class StdComponentCreator(object):

    # constructor
    def __init__(self, configServer, parent):
        # configuration server
        self.configServer = configServer

        # created components
        self.components = {}
        # created components
        self.datasources = {}
        # parent
        self.parent = parent

    # creates component and datasources from online xml
    def create(self):
        self.action = None
        self.componentName = None
        self.components = {}
        self.datasources = {}
        if NXSTOOLS_AVAILABLE:
            options = Options(self.configServer.getDeviceName())

            scpc = StandardCPCreator(options, [], False)
            self.action = self.selectComponent(scpc)
            if self.action:
                scpc.createXMLs()
                self.components = dict(scpc.components)
                self.datasources = dict(scpc.datasources)

    # runs Creator widget to select the required component
    # \returns action to be performed with the components and datasources
    def selectComponent(self, scpc):
        aform = StdCreatorDlg(scpc, self.parent)
        action = None
        aform.createGUI()
        aform.show()
        if aform.exec_():
            action = aform.action
        return action


# configuration server
class DataSourceCreator(object):

    # constructor
    def __init__(self, configServer, parent):
        # configuration server
        self.configServer = configServer

        # online file name
        self.onlineFile = None

        # componentName
        self.componentName = None

        # created components
        self.datasources = {}
        # available components
        self.avcomponents = []
        # parent
        self.parent = parent

    # sets onlineFile name and check if online file exists
    def checkOnlineFile(self, onlineFile):
        onlineFile = onlineFile or '/online_dir/online.xml'
        onlineFile = unicode(QFileDialog.getOpenFileName(
            self.parent, "Open Online File", onlineFile,
            "XML files (*.xml)")[0])
        if onlineFile:
            self.onlineFile = onlineFile
            return True
        else:
            self.onlineFile = None

    # creates component and datasources from online xml
    def create(self):
        self.action = None
        self.datasources = {}
        if NXSTOOLS_AVAILABLE and self.onlineFile:
            options = Options(self.configServer.getDeviceName())

            occ = OnlineDSCreator(options, [self.onlineFile], False)
            self.action = self.selectAction()
            if self.action:
                occ.createXMLs()
                self.datasources = dict(occ.datasources)

    # runs Creator widget to select the required component
    # \returns action to be performed with the components and datasources
    def selectAction(self):
        aform = CreatorDlg(self.parent)
        action = None
        aform.createGUI()
        aform.setWindowTitle("DataSource Creator")
        aform.ui.compFrame.hide()
        aform.resize(600, 0)
        aform.show()
        if aform.exec_():
            action = aform.action
        return action


# test function
def test():
    import sys
    from PyQt5.QtGui import QApplication

    app = QApplication(sys.argv)
    app.exit()


if __name__ == "__main__":
    test()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file GroupDlg.py
# Group dialog class

""" group widget """

import copy
import os
import sys

from PyQt5.QtCore import (Qt, QModelIndex)
from PyQt5.QtWidgets import (QMessageBox, QTableWidgetItem, QCompleter)
from PyQt5 import uic

# from .ui.ui_groupdlg import Ui_GroupDlg
from .AttributeDlg import AttributeDlg
from .NodeDlg import NodeDlg
from .DomTools import DomTools

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "groupdlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a group tag
class GroupDlg(NodeDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(GroupDlg, self).__init__(parent)

        # group name
        self.name = u''
        # group type
        self.nexusType = u''
        # group doc
        self.doc = u''
        # group attributes
        self.attributes = {}
        # group attributes
        self.__attributes = {}

        # allowed subitems
        self.subItems = ["group", "field", "attribute", "link",
                         "component", "doc"]

        # list of NeXus types
        self.typehelper = [
            'NXaperture',
            'NXattenuator',
            'NXbeam',
            'NXbeam_stop',
            'NXbending_magnet',
            'NXcapillary',
            'NXcharacterization',
            'NXcite',
            'NXcollection',
            'NXcollimator',
            'NXcrystal',
            'NXdata',
            'NXdetector_group',
            'NXdetector_module',
            'NXdetector',
            'NXdisk_chopper',
            'NXentry',
            'NXenvironment',
            'NXevent_data',
            'NXfermi_chopper',
            'NXfilter',
            'NXflipper',
            'NXfresnel_zone_plate',
            'NXgeometry',
            'NXgrating',
            'NXguide',
            'NXinsertion_device',
            'NXinstrument',
            'NXlog',
            'NXmirror',
            'NXmoderator',
            'NXmonitor',
            'NXmonochromator',
            'NXnote',
            'NXobject',
            'NXorientation',
            'NXparameters',
            'NXpinhole',
            'NXpolarizer',
            'NXpositioner',
            'NXprocess',
            'NXroot',
            'NXsample',
            'NXsensor',
            'NXshape',
            'NXslit',
            'NXsource',
            'NXsubentry',
            'NXtransformations',
            'NXtranslation',
            'NXuser',
            'NXvelocity_selector',
            'NXxraylens'
        ]

        # user interface
        self.ui = _formclass()

    # updates the group dialog
    # \brief It sets the form local variables
    def updateForm(self):

        if self.name is not None:
            self.ui.nameLineEdit.setText(self.name)
        if self.nexusType is not None:
            self.ui.typeLineEdit.setText(self.nexusType)
        if self.doc is not None:
            self.ui.docTextEdit.setText(self.doc)

        self.__attributes.clear()
        for at in self.attributes.keys():
            self.__attributes[unicode(at)] = self.attributes[(unicode(at))]

        self.populateAttributes()

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):
        self.ui.setupUi(self)
        completer = QCompleter(
            [str(tp) for tp in self.typehelper],
            self)
        self.ui.typeLineEdit.setCompleter(completer)
        self.updateForm()

        self.__updateUi()

        self.ui.resetPushButton.clicked.connect(self.reset)
        self.ui.attributeTableWidget.itemChanged.connect(
            self.__tableItemChanged)
        self.ui.addPushButton.clicked.connect(self.__addAttribute)
        self.ui.removePushButton.clicked.connect(self.__removeAttribute)

        self.ui.typeLineEdit.textEdited[str].connect(self.__updateUi)

    # provides the state of the group dialog
    # \returns state of the group in tuple
    def getState(self):
        attributes = copy.copy(self.attributes)

        state = (self.name,
                 self.nexusType,
                 self.doc,
                 attributes
                 )
        return state

    # sets the state of the group dialog
    # \param state group state written in tuple
    def setState(self, state):

        (self.name,
         self.nexusType,
         self.doc,
         attributes
         ) = state
        self.attributes = copy.copy(attributes)

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if node:
            # defined in NodeDlg class
            self.node = node
        if not self.node:
            # exception?
            return
        attributeMap = self.node.attributes()

        self.name = unicode(
            attributeMap.namedItem("name").nodeValue()
            if attributeMap.contains("name") else "")
        self.nexusType = unicode(
            attributeMap.namedItem("type").nodeValue()
            if attributeMap.contains("type") else "")

        self.attributes.clear()
        self.__attributes.clear()
        for i in range(attributeMap.count()):
            attribute = attributeMap.item(i)
            attrName = unicode(attribute.nodeName())
            if attrName != "name" and attrName != "type":
                self.attributes[attrName] = unicode(attribute.nodeValue())
                self.__attributes[attrName] = unicode(attribute.nodeValue())

        doc = self.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.doc = unicode(text).strip() if text else ""

    # adds an attribute
    #  \brief It runs the Group Dialog and fetches attribute name and value
    def __addAttribute(self):
        aform = AttributeDlg()
        if aform.exec_():
            if aform.name not in self.__attributes.keys():
                self.__attributes[aform.name] = aform.value
                self.populateAttributes(aform.name)
            else:
                QMessageBox.warning(
                    self, "Attribute name exists",
                    "To change the attribute value, please edit the value "
                    "in the attribute table")

    # takes a name of the current attribute
    # \returns name of the current attribute
    def __currentTableAttribute(self):
        item = self.ui.attributeTableWidget.item(
            self.ui.attributeTableWidget.currentRow(), 0)
        if item is None:
            return None
        return item.data(Qt.UserRole)

    # removes an attribute
    #  \brief It removes the current attribute asking before about it
    def __removeAttribute(self):
        attr = self.__currentTableAttribute()
        if attr is None:
            return
        if unicode(attr) in self.__attributes.keys():
            self.__attributes.pop(unicode(attr))
            self.populateAttributes()

    # changes the current value of the attribute
    # \brief It changes the current value of the attribute
    #        and informs the user that attribute names arenot editable
    def __tableItemChanged(self, item):
        attr = self.__currentTableAttribute()
        if unicode(attr) not in self.__attributes.keys():
            return
        column = self.ui.attributeTableWidget.currentColumn()
        if column == 1:
            self.__attributes[unicode(attr)] = unicode(item.text())
        if column == 0:
            QMessageBox.warning(
                self, "Attribute name is not editable",
                "To change the attribute name, "
                "please remove the attribute and add the new one")
        self.populateAttributes()

    # fills in the attribute table
    # \param selectedAttribute selected attribute
    def populateAttributes(self, selectedAttribute=None):
        selected = None
        self.ui.attributeTableWidget.clear()
        self.ui.attributeTableWidget.setSortingEnabled(False)
        self.ui.attributeTableWidget.setRowCount(len(self.__attributes))
        headers = ["Name", "Value"]
        self.ui.attributeTableWidget.setColumnCount(len(headers))
        self.ui.attributeTableWidget.setHorizontalHeaderLabels(headers)
        for row, name in enumerate(self.__attributes):
            item = QTableWidgetItem(name)
            item.setData(Qt.UserRole, (name))
            self.ui.attributeTableWidget.setItem(row, 0, item)
            item2 = QTableWidgetItem(self.__attributes[name])
            self.ui.attributeTableWidget.setItem(row, 1, item2)
            if selectedAttribute is not None and selectedAttribute == name:
                selected = item2
        self.ui.attributeTableWidget.setSortingEnabled(True)
        self.ui.attributeTableWidget.resizeColumnsToContents()
        self.ui.attributeTableWidget.horizontalHeader()\
            .setStretchLastSection(True)
        if selected is not None:
            selected.setSelected(True)
            self.ui.attributeTableWidget.setCurrentItem(selected)

    # updates group user interface
    # \brief It sets enable or disable the OK button
    def __updateUi(self):
        enable = bool(self.ui.typeLineEdit.text())
        self.ui.applyPushButton.setEnabled(enable)

    # applys input text strings
    # \brief It copies the group name and type from lineEdit widgets
    #        and apply the dialog
    def apply(self):
        self.name = unicode(self.ui.nameLineEdit.text())
        self.nexusType = unicode(self.ui.typeLineEdit.text())

        self.doc = unicode(self.ui.docTextEdit.toPlainText())

        index = self.view.currentIndex()
        finalIndex = self.view.model().createIndex(
            index.row(), 2, index.parent().internalPointer())

        self.attributes.clear()
        for at in self.__attributes.keys():
            self.attributes[at] = self.__attributes[at]

        self.view.expand(index)

        if self.node and self.root and self.node.isElement():
            self.updateNode(index)
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.view.expand(index)
        self.view.model().dataChanged.emit(index, finalIndex)

    # updates the Node
    # \brief It sets node from the dialog variables
    def updateNode(self, index=QModelIndex()):
        elem = self.node.toElement()
        mindex = self.view.currentIndex() if not index.isValid() else index

        attributeMap = self.node.attributes()
        for _ in range(attributeMap.count()):
            attributeMap.removeNamedItem(attributeMap.item(0).nodeName())
        if self.name:
            elem.setAttribute(str("name"), str(self.name))
        if self.nexusType:
            elem.setAttribute(str("type"), str(self.nexusType))

        for attr in self.attributes.keys():
            elem.setAttribute(str(attr), str(self.attributes[attr]))

        doc = self.node.firstChildElement(str("doc"))
        if not self.doc and doc and doc.nodeName() == "doc":
            self.removeElement(doc, mindex)
        elif self.doc:
            newDoc = self.root.createElement(str("doc"))
            newText = self.root.createTextNode(str(self.doc))
            newDoc.appendChild(newText)
            if doc and doc.nodeName() == "doc":
                self.replaceElement(doc, newDoc, mindex)
            else:
                self.appendElement(newDoc, mindex)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # group form
    form = GroupDlg()
    form.name = 'entry'
    form.nexusType = 'NXentry'
    form.doc = 'The main entry'
    form.attributes = {"title": "Test run 1", "run_cycle": "2012-1"}
    form.createGUI()
    form.show()
    app.exec_()

    if form.nexusType:
        logger.info("Group: name = \'%s\' type = \'%s\'" % (
            form.name, form.nexusType))
    if form.attributes:
        logger.info("Other attributes:")
        for k in form.attributes.keys():
            logger.info(" %s = '%s' " % (k, form.attributes[k]))
    if form.doc:
        logger.info("Doc: \n%s" % form.doc)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ItemSlots.py
# user pool commands of GUI application

""" Item slots """

from PyQt5.QtGui import (QKeySequence)
from PyQt5.QtWidgets import (QMessageBox)
from PyQt5.QtCore import (Qt)


from .ItemCommands import (
    ComponentClear,
    ComponentLoadComponentItem,
    ComponentRemoveItem,
    ComponentCopyItem,
    ComponentPasteItem,
    CutItem,
    CopyItem,
    PasteItem,
    ComponentMerge,
    ComponentNewItem,
    ComponentLoadDataSourceItem,
    ComponentAddDataSourceItem,
    ComponentLinkDataSourceItem,
    ComponentApplyItem,
    ComponentMoveUpItem,
    ComponentMoveDownItem
)

from .EditCommands import (
    ComponentEdit
)

from .DataSourceDlg import CommonDataSourceDlg
from .ComponentDlg import ComponentDlg


# stack with the application commands
class ItemSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # action data
        self.actions = {
            "actionNewGroupItem": [
                "Add &Group Item", "componentNewGroupItem",
                "", "componentnewitem", "Add a new component group item"],
            "actionNewFieldItem": [
                "Add &Field Item", "componentNewFieldItem",
                "", "componentnewitem", "Add a new  component field item"],
            "actionNewStrategyItem": [
                "Add &Strategy Item", "componentNewStrategyItem",
                "", "componentnewitem", "Add a new component strategy item"],
            "actionNewDataSourceItem": [
                "Add &DataSource Item", "componentNewDataSourceItem",
                "", "componentnewitem",
                "Add a new component data source item"],
            "actionNewAttributeItem": [
                "Add A&ttribute Item", "componentNewAttributeItem",
                "", "componentnewitem", "Add a new component attribute item"],
            "actionNewLinkItem": [
                "Add &Link Item", "componentNewLinkItem",
                "", "componentnewitem", "Add a new  component link item"],
            "actionLoadSubComponentItem": [
                "Load SubComponent Item...", "componentLoadComponentItem",
                "", "componentloaditem",
                "Load an existing component part from the file"],
            "actionLoadDataSourceItem": [
                "Load DataSource Item...", "componentLoadDataSourceItem",
                "", "componentloaditem",
                "Load an existing data source from the file"],
            "actionAddDataSourceItem": [
                "Append DataSource Item", "componentAddDataSourceItem",
                QKeySequence(Qt.CTRL + Qt.Key_Plus),
                "componentadditem", "Append the data source from the list"],
            "actionLinkDataSourceItem": [
                "Link DataSource Item", "componentLinkDataSourceItem",
                "Ctrl+L",
                "componentlinkitem", "Link the data source from the list"],
            "actionMoveUpComponentItem": [
                "&Move Up Component Item", "componentMoveUpItem",
                "Ctrl+[", "componentsmoveupitem",
                "Move up the component item"],
            "actionMoveDownComponentItem": [
                "&Move Down Component Item", "componentMoveDownItem",
                "Ctrl+]", "componentsmovedownitem",
                "Move down the component item"],
            "actionMergeComponentItems": [
                "Merge Component Items", "componentMerge",
                "Ctrl+M", "componentmerge", "Merge the component items"],
            "actionApplyComponentItem": [
                "&Apply Component Item", "componentApplyItem",
                "Ctrl+R", "componentsapplyitem", "Apply the component item"],
            "actionClearComponentItems": [
                "Clear Component Items", "componentClear",
                "", "componentclear", "Removes all component items"],
            "actionCutItem": [
                "C&ut Item", "cutItem",
                QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_X),
                "cut", "Cut the item"],
            "actionCopyItem": [
                "&Copy Item", "copyItem",
                QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_C),
                "copy", "Copy the item"],
            "actionPasteItem": [
                "&Paste Item", "pasteItem",
                QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_V),
                "paste", "Paste the item"],
            "actionCutComponentItem": [
                "Cut Component Item", "componentRemoveItem",
                QKeySequence(Qt.CTRL + Qt.Key_Delete),
                "cut", "Remove the component item"],
            "actionCopyComponentItem": [
                "Copy Component Item", "componentCopyItem", "",
                "copy", "Copy the component item"],
            "actionPasteComponentItem": [
                "Paste Component Item", "componentPasteItem",
                QKeySequence(Qt.CTRL + Qt.Key_Insert),
                "paste", "Paste the component item"],
        }

    # copy component item action
    # \brief It copies the  current component item into the clipboard
    def componentCopyItem(self):
        cmd = ComponentCopyItem(self.main)
        cmd.redo()

    # remove component item action
    # \brief It removes the current component item and copies it
    #        into the clipboard
    def componentRemoveItem(self):
        cmd = ComponentRemoveItem(self.main)
        self.undoStack.push(cmd)

    # paste component item action
    # \brief It pastes the component item from the clipboard
    def componentPasteItem(self):
        cmd = ComponentPasteItem(self.main)
        self.undoStack.push(cmd)

    # copy item action
    # \brief It copies the current item into the clipboard
    def copyItem(self):
        cmd = CopyItem(self.main)
        if self.main.ui.mdi.activeSubWindow() and isinstance(
                self.main.ui.mdi.activeSubWindow().widget(), ComponentDlg):
            cmd.type = "component"
        elif self.main.ui.mdi.activeSubWindow() and isinstance(
                self.main.ui.mdi.activeSubWindow().widget(),
                CommonDataSourceDlg):
            cmd.type = "datasource"
        else:
            QMessageBox.warning(self.main, "Item not selected",
                                "Please select one of the items")
            cmd.type = None
            return
        cmd.redo()

    # cuts item action
    # \brief It removes the current item and copies it into the clipboard
    def cutItem(self):
        cmd = CutItem(self.main)
        if self.main.ui.mdi.activeSubWindow() and isinstance(
                self.main.ui.mdi.activeSubWindow().widget(), ComponentDlg):
            cmd.type = "component"
        elif self.main.ui.mdi.activeSubWindow() and isinstance(
                self.main.ui.mdi.activeSubWindow().widget(),
                CommonDataSourceDlg):
            cmd.type = "datasource"
        else:
            QMessageBox.warning(self.main, "Item not selected",
                                "Please select one of the items")
            cmd.type = None

            return
        self.undoStack.push(cmd)

    # paste item action
    # \brief It pastes the item from the clipboard
    def pasteItem(self):
        cmd = PasteItem(self.main)
        if self.main.ui.mdi.activeSubWindow() and isinstance(
                self.main.ui.mdi.activeSubWindow().widget(), ComponentDlg):
            cmd.type = "component"
        elif self.main.ui.mdi.activeSubWindow() and isinstance(
                self.main.ui.mdi.activeSubWindow().widget(),
                CommonDataSourceDlg):
            cmd.type = "datasource"
        else:
            QMessageBox.warning(self.main, "Item not selected",
                                "Please select one of the items")
            cmd.type = None
            return
        self.undoStack.push(cmd)

    # new group component item action
    # \brief It adds a new group component item
    def componentNewGroupItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentNewItem(self.main)
            cmd.itemName = 'group'
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # new group component item action
    # \brief It adds a new group component item
    def componentNewStrategyItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentNewItem(self.main)
            cmd.itemName = 'strategy'
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # new field component item action
    # \brief It adds a new field component item
    def componentNewFieldItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentNewItem(self.main)
            cmd.itemName = 'field'
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # new attribute component item action
    # \brief It adds a new attribute component item
    def componentNewAttributeItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentNewItem(self.main)
            cmd.itemName = 'attribute'
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # new link component item action
    # \brief It adds a new link component item
    def componentNewLinkItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentNewItem(self.main)
            cmd.itemName = 'link'
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # new datasource component item action
    # \brief It adds a new datasource component item
    def componentNewDataSourceItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentNewItem(self.main)
            cmd.itemName = 'datasource'
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # load sub-component item action
    # \brief It loads a sub-component item from a file
    def componentLoadComponentItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentLoadComponentItem(self.main)
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # load datasource component item action
    # \brief It loads a datasource component item from a file
    def componentLoadDataSourceItem(self):
        if hasattr(self.main.ui.mdi.activeSubWindow(), 'widget') and \
                isinstance(self.main.ui.mdi.activeSubWindow().widget(),
                           ComponentDlg):
            cmd = ComponentLoadDataSourceItem(self.main)
            self.undoStack.push(cmd)
        else:
            QMessageBox.warning(self.main, "Component not created",
                                "Please edit one of the components")

    # add datasource component item action
    # \brief It adds the current datasource item into component tree
    def componentAddDataSourceItem(self):
        cmd = ComponentAddDataSourceItem(self.main)
        self.undoStack.push(cmd)

    # link datasource component item action
    # \brief It adds the current datasource item into component tree
    def componentLinkDataSourceItem(self):
        cmd = ComponentLinkDataSourceItem(self.main)
        self.undoStack.push(cmd)

    # link datasource component item action
    # \brief It adds the current datasource item into component tree
    def componentLinkDataSourceItemButton(self):
        if self.main.updateComponentListItem():
            self.componentLinkDataSourceItem()

    # move-up component item action
    # \brief It moves the current component item up
    def componentMoveUpItem(self):
        cmd = ComponentMoveUpItem(self.main)
        self.undoStack.push(cmd)

    # move-down component item action
    # \brief It moves the current component item down
    def componentMoveDownItem(self):
        cmd = ComponentMoveDownItem(self.main)
        self.undoStack.push(cmd)

    # apply component item action
    # \brief It applies the changes in the current component item
    def componentApplyItem(self):
        cmd = ComponentApplyItem(self.main)
        self.undoStack.push(cmd)

    # apply component item action executed by button
    # \brief It applies the changes in the current component item
    #        executed by button
    def componentApplyItemButton(self):
        if self.main.updateComponentListItem():
            self.componentApplyItem()

    # merge component action
    # \brief It merges the current component
    def componentMerge(self):
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentMerge(self.main)
        self.undoStack.push(cmd)

    # clear component action
    # \brief It clears the current component
    def componentClear(self):
        cmd = ComponentClear(self.main)
        self.undoStack.push(cmd)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file Merger.py
# Class for merging DOM component trees

""" merger of different configuration trees """

import sys

from PyQt5.QtXml import QDomNode


from PyQt5.QtCore import QThread
from PyQt5.QtWidgets import QDialog, QLabel, QVBoxLayout, QPushButton

from .Errors import IncompatibleNodeError
from .DomTools import DomTools

if sys.version_info > (3,):
    unicode = str


# dialog of the merger
class MergerDlg(QDialog):

    # constructor
    # \param parent dialog
    def __init__(self, parent=None):
        super(MergerDlg, self).__init__(parent)

        # interrupt button
        self.interruptButton = None

    # creates GUI
    # \brief It creates dialog with a merging label
    def createGUI(self):
        label = QLabel(" Please be patient: Component merging...")
        # button to interrupt merging
        self.interruptButton = QPushButton("&Interrupt Merging")
        layout = QVBoxLayout()
        layout.addWidget(label)
        layout.addWidget(self.interruptButton)
        self.setLayout(layout)
        self.setWindowTitle("Merging")


# merges the components
class Merger(QThread):

    # destructor waiting for thread
    def __del__(self):
        self.wait()

    # constructor
    # \param root  DOM root node
    def __init__(self, root):
        QThread.__init__(self)
        # DOM root node
        self._root = root
        # tags which cannot have the same siblings
        self._singles = ['strategy', 'dimensions', 'definition',
                         'record', 'device', 'query', 'database']

        # allowed children of the given nodes
        self._children = {
            "attribute": ["datasource", "strategy", "enumeration", "doc",
                          "dimensions"],
            "definition": ["group", "field", "attribute", "link", "component",
                           "doc", "symbols"],
            "dimensions": ["dim", "doc"],
            "field": ["attribute", "datasource", "doc", "dimensions",
                      "enumeration", "strategy"],
            "group": ["group", "field", "attribute", "link", "component",
                      "doc"],
            "link": ["datasource", "strategy", "doc"],
            "dim": ["datasource", "strategy", "doc"]
        }

        # with unique text
        self.uniqueText = ['field', 'attribute', 'query', 'strategy', 'result']

        # required attributes
        self._requiredAttr = {
            "attribute": ["name"],
            "definition": [],
            "dimensions": ["rank"],
            "dim": ["index"],
            "field": ["name"],
            "group": ["type"],
            "link": ["name"],
            "strategy": ["mode"],
            "datasource": ["type"],
            "record": ["name"],
            "device": ["member", "name"],
            "database": ["dbtype"],
            "query": ["format"]
        }

        # it contains an exception instance when the exception was raised
        self.exception = None
        # it has to be set on False when we want to break merging
        self.running = True
        # selected node
        self.selectedNode = None

    # fetches node ancestors int the tree
    # \param node the given DOM node
    # \returns string with node ancestors in the tree
    def _getAncestors(self, node):
        res = ""
        attr = node.attributes()

        name = attr.namedItem("name").nodeValue() \
            if attr.contains("name") else ""

        if node.parentNode().nodeName() != '#document':
            res = self._getAncestors(node.parentNode())
        res += "/" + unicode(node.nodeName())
        if name:
            res += ":" + name
        return res

    # checks if the given node elements are mergeable
    # \param elem1 first node element
    # \param elem2 secound node element
    # \returns True if the given elements are mergeable
    def _areMergeable(self, elem1, elem2):
        if elem1.nodeName() != elem2.nodeName():
            return False
        tagName = unicode(elem1.nodeName())
        attr1 = elem1.attributes()
        attr2 = elem2.attributes()
        status = True
        tags = []

        name1 = attr1.namedItem("name").nodeValue() \
            if attr1.contains("name") else ""
        name2 = attr2.namedItem("name").nodeValue() \
            if attr1.contains("name") else ""

        if name1 != name2 and name1 and name2:
            if tagName in self._singles:
                raise IncompatibleNodeError(
                    "Incompatible element attributes  %s: " % unicode(tags),
                    [elem1, elem2])
            return False

        for i1 in range(attr1.count()):
            at1 = attr1.item(i1)
            for i2 in range(attr2.count()):
                at2 = attr2.item(i2)
                if at1.nodeName() == at2.nodeName() and \
                        at1.nodeValue() != at2.nodeValue():
                    status = False
                    tags.append((unicode(self._getAncestors(at1)),
                                 unicode(at1.nodeValue()),
                                 unicode(at2.nodeValue())))

        if not status and (tagName in self._singles
                           or (name1 and name1 == name2)):
            raise IncompatibleNodeError(
                "Incompatible element attributes  %s: " % unicode(tags),
                [elem1, elem2])

        if tagName in self.uniqueText:
            text1 = unicode(DomTools.getText(elem1)).strip()
            text2 = unicode(DomTools.getText(elem2)).strip()

            if text1 != text2 and text1 and text2:
                raise IncompatibleNodeError(
                    "Incompatible \n%s element value\n%s \n%s "
                    % (unicode(self._getAncestors(elem1)), text1, text2),
                    [elem1, elem2])

        return status

    # merges the given node elements
    # \param elem1 first node element
    # \param elem2 secound node element
    def _mergeNodes(self, elem1, elem2):
        attr2 = elem2.attributes()
        texts = []

        for i2 in range(attr2.count()):
            at2 = attr2.item(i2)
            elem1.setAttribute(at2.nodeName(), at2.nodeValue())

        child1 = elem1.firstChild()
        while not child1.isNull():
            if child1.nodeType() == QDomNode.TextNode:
                texts.append(unicode(child1.toText().data()).strip())
            child1 = child1.nextSibling()

        toMove = []

        child2 = elem2.firstChild()
        while not child2.isNull():
            if child2.nodeType() == QDomNode.TextNode:
                if unicode(child2.toText().data()).strip() not in texts:
                    toMove.append(child2)
            else:
                toMove.append(child2)
            child2 = child2.nextSibling()

        for child in toMove:
            elem1.appendChild(child)
        toMove = []

        parent = elem2.parentNode()
        if self.selectedNode == elem2:
            self.selectedNode = elem1
        parent.removeChild(elem2)

    # checks if the nodes has required attributes
    # \param node the given DOM node
    def _hasAttributes(self, node):
        elem1 = node.toElement()
        if elem1 is not None:
            attr1 = elem1.attributes()
            name1 = attr1.namedItem("name").nodeValue() \
                if attr1.contains("name") else ""
            if elem1.nodeName() in self._requiredAttr.keys():
                for at1 in self._requiredAttr[unicode(elem1.nodeName())]:
                    if not attr1.contains(at1) \
                            or not unicode(attr1.namedItem(at1).nodeValue()
                                           ).strip():
                        message = "Not defined %s attribute of %s%s " \
                            % (at1, unicode(elem1.nodeName()),
                               (":" + unicode(name1))
                               if unicode(name1).strip() else " ")
                        raise IncompatibleNodeError(message, [elem1])

    # merge all children of the given DOM node
    # \param node the given DOM node
    def _mergeChildren(self, node):
        if node:
            self._hasAttributes(node)

            children = node.childNodes()
            nchildren = children.count()
            c1 = 0
            while c1 < nchildren:
                child1 = children.item(c1)
                elem1 = child1.toElement()
                c2 = c1 + 1
                while c2 < nchildren:
                    child2 = children.item(c2)
                    if child1 != child2:
                        elem2 = child2.toElement()
                        if elem1 is not None and elem2 is not None:
                            if self._areMergeable(elem1, elem2):
                                self._mergeNodes(elem1, elem2)
                                nchildren -= 1
                                c2 -= 1
                    c2 += 1
                c1 += 1

            child = node.firstChild()
            elem = node.toElement()
            nName = unicode(elem.nodeName()) if elem else ""

            if elem and nName == 'query':
                if not unicode(elem.text()).strip():
                    raise IncompatibleNodeError(
                        "Empty content of the query tag: %s"
                        % (self._getAncestors(elem)),
                        [elem])

            if child:
                while not child.isNull() and self.running:
                    if nName and nName in self._children.keys():
                        childElem = child.toElement()
                        cName = unicode(childElem.nodeName()) \
                            if childElem else ""
                        if cName and cName not in self._children[nName]:
                            raise IncompatibleNodeError(
                                "Not allowed <%s> child of \n < %s > \n parent"
                                % (cName, self._getAncestors(elem)),
                                [childElem])

                    self._mergeChildren(child)
                    child = child.nextSibling()

    # runs thread
    # \brief It runs the mergeChildren with the root node and catches
    #        the exceptions if needed
    def run(self):
        if self._root:
            try:
                self._mergeChildren(self._root)
                if not self.running:
                    raise Exception("Merging Interrupted")
            except Exception as e:
                self.exception = e

        self.finished.emit()


if __name__ == "__main__":
    pass
